#  Copyright (C) 2014, 2016  Chris Gerrard
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

require 'nokogiri'
require 'digest/md5'

module Twb

  class Action < TabClass

    # Notes
    # Sources:
    # <source>
    #   type="datasource"
    #     <source datasource="Sample - Superstore - English (Extract)" type="datasource">
    #       <exclude-sheets  - use to exclude these from the Workbook's sheets
    #                        - haven't seen an example of named sheet for datasoure
    #                        - can only target a single worksheet, by observation
    #   type="sheet"
    #     Worksheet
    #     <source type="sheet" worksheet="Sheet 3"/>
    #                        - use the worksheet attribute
    #                        - can only target a single worksheet, by observation
    #     Dashboard
    #     <source dashboard="Dashboard 1" type="sheet" worksheet="Sheet 5"/>
    #                        - dashboard attribute is present
    #                        - use the worksheet attribute, if available
    #     <source dashboard="Dashboard 2" type="sheet"/>
    #       NO <exclude-sheets - source is all of the dashboard's sheets
 
    @@hasher = Digest::SHA256.new

    attr_reader :workbook, :node 
    attr_reader :name, :caption, :uiname
    attr_reader :type
    attr_reader :sourceDash, :sourceType, :sourceSheet
    attr_reader :cmdTarget,  :cmdExclude, :cmdSpecFields
    attr_reader :linkExpression

    def initialize actionNode, workbookNode
      @workbook  = workbookNode
      @node      = actionNode
      # --
      @name      = @node.attr('name')
      @caption   = @node.attr('caption')
      @uiname    = @caption.nil? ? @name : @caption
      # --
      @type      = setType
      # --
      @cmdTarget     = cmdParam 'target'
      @cmdExclude    = cmdParam 'exclude'
      @cmdSpecFields = cmdParam 'special-fields'
      # --
      @linkExpression = linkExpr
      process
      return self
    end

    def id
        @id ||= @id = @name.hash
    end

    def process

    end

    def setType
      linkNode = @node.at_xpath('./link')
      return :link unless linkNode.nil?
      command  = @node.at_xpath('./command/@command')
      return :notset if command.nil?
      type = case command.text
        when 'tsc:brush'
          :highlight
        when 'tsc:tsl-filter'
          :filter
        else
          command.text
      end
    end

    def linkExpr
      attr = @node.at_xpath('./link/@expression')
      attr.nil? ? nil : attr.text
    end

    def cmdParam param
      n = @node.at_xpath("./command/param[@name='#{param}']")
      return nil if n.nil?
      v = n.attribute('value')
      v.nil? ? nil : v.text
    end

    def sourceDash
      attr = @node.at_xpath('./source/@dashboard')
      attr.nil? ? nil : attr.text
    end

    def sourceSheet
      attr = @node.at_xpath('./source/@worksheet')
      attr.nil? ? nil : attr.text
    end

    def sourceType
      @node.at_xpath('./source/@type')
    end

    def typeConv rawType
    end

  end

end
