#  DocumentedFieldsCSVEmitter.rb - this Ruby script Copyright 2017 Christopher Gerrard
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

require 'twb'
require 'csv'

module Twb
module Analysis

  class AnnotatedFieldsCSVEmitter

    attr_reader :csvFileName, :csvRecords
    attr_reader :dsCount,     :fieldsCount  

    @@csvHeader = ['Record #',
                   'Workbook',           
                   'Workbook Dir',
                   'Data Source',
                   'Field',
                   'Doc Type',
                   'Doc Line #',
                   'Doc Line'
                  ]
    @@csvFileType = 'TwbAnnotatedFields'
    @@csvFileName = @@csvFileType + '.csv'

    def initialize
      # puts "AnnotatedFieldsCSVEmitter - initializing"
      @csvFile     = CSV.open(@@csvFileName,'w')
      @csvFile     << @@csvHeader
      @csvRecords  = Set.new
      # --
      @recNum    = 0
      @dsCount   = 0
      @fieldNum  = 0
    end

    def self.csvHeader
      @@csvHeader
    end

    def self.csvFileType
      @@csvFileType
    end

    def self.csvFileNaame
      @@csvFileName
    end

    def processTwb twb
      @twb = nil
      @twb         = twb                    if twb.instance_of? Twb::Workbook
      @twb         = Twb::Workbook.new(twb) if twb.instance_of? String
      raise ArgumentError.new("ERROR in Workbok processing: '#{twb}' must be a Workbook (class) or the name of a Workbook (String), is a #{twb.class} \n ") unless @twb.is_a? Twb::Workbook
      # puts "Processing #{@twb.name}"
      # --
      dss = @twb.datasources
      dss.each do |ds|
        @dsname = ds.uiname
        # puts "\n    -- #{ds.uiname} "
        # tables = Set.new
        fields = {}
        @dsCount += 1
        ds.columnFields.each do |field|
          recordField field
        end
      end
    end # def processTwb twb

    def recordField field
      # puts "- #{field.uiname} :: #{field} ::: #{field.node}"
      @fieldNum += 1
      @lineNum   = 0
      field.comment.each do |line|
        # dispLine = if ''.eql?(line) the ? '' : line
        @csvFile << [ @recNum+=1,
                      @twb.name,
                      @twb.dir,
                      @dsname,
                      field.uiname,
                      'Comment',
                      @lineNum+=1,
                      # ''.eql?(line) ? " " : line
                      line
                    ]
      end
      unless field.calcField.nil?
        @lineNum   = 0
        field.calcField.formulaResolvedLines.each do |line|
          @csvFile << [ @recNum+=1,
                        @twb.name,
                        @twb.dir,
                        @dsname,
                        field.uiname,
                        'Formula',
                        @lineNum+=1,
                        line
                      ]
        end
      end
    end

    # def recordField fields, field, props
    #   # puts "%-65s  :: %s " % [fieldName,props]
    #   return if field.uiname.nil?
    #   if fields.has_key? field.uiname
    #     fields[field.uiname].merge! field.properties
    #   else
    #     fields[field.uiname] = field.properties
    #   end
    # end

    def emitFields fields
      fields.each do |fieldName,props|
        # puts "FIELD::  %-40s  :: %s" % [fieldName,props.inspect]
        # class = props[]
        csvRec = [ @recNum+=1,
                   @twb.name,
                   @twb.dir,
                   @dsname,
                   fieldName,
                   props[:type],
                   props['hidden'],
                   props[:columnField],
                   props[:calculatedField],
                   props[:dbField],
                   props[:mappedField],
                   props[:metadataField]
                 ]
        @csvFile << csvRec
        @csvRecords.add csvRec
      end
    end

    def recordTech field, type
      @recNum+=1
      field.properties.each do |name,value|
        @csvFileTech << [ @recNum,
                          @twb.name,
                          @twb.dir,
                          @dsname,
                          field.uiname,
                          type,
                          name,
                          value
                        ]
      end
    end

    def emitTech dataSource, fieldName, type, properties
      # puts "XX #{dataSource.uiname} :: #{fieldName} #{}  :: #{type} :: #{properties}"
      @recNum+=1
      properties.each do |name,value|
        @csvFileTech << [ @recNum,
                          @twb.name,
                          @twb.dir,
                          dataSource.uiname,
                          fieldName,
                          type,
                          name,
                          value
                        ]
      end
    end

    def cleanup
      @csvFile.close unless @csvFile.nil?
    end

  end # class DocumentedFieldsCSVEmitter

end   # module Analysis
end   # module Twb