#  calculatedfieldsanalyzer.rb - this Ruby script Copyright 2017 Christopher Gerrard
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

require 'twb'

module Twb
module Analysis
module CalculatedFields

  class CSVEmitter

    attr_reader :calcFieldsType, :calcFieldsHeader,  :calcFieldsFile,  :calcFieldsRecords, :calcFieldsCount
    attr_reader :refFieldsType,  :refFieldsHeader,   :refFieldsFile,   :refFieldsRecords,  :refFieldsCount
    attr_reader :dictionary,     :csvFiles

    def initialize
      @calcFieldsContents = 'Calculated Fields with Formulas'
      @calcFieldsType     = 'CalculatedFieldsWithFormulas'
      @calcFieldsHeader   = ['Record #',
                             'Workbook',           'Workbook Dir',
                             'Data Source Name (tech)', 'Data Source Caption', 'Data Source Name',
                             'Field Name (tech)',  'Field Caption',       'Field Name',
                             'Data Source + Field Name (tech)',
                             'Formula (flat)',
                             'Formula Length',
                             'Formula Table Calc?',
                             'Formula LOD?',
                             'Formula LOD Code Position'
                            ]

      @refFieldsContents = 'Fields referenced in Calculated Fields Formulae'
      @refFieldsType     = 'CalculatedFieldsReferencedFields'
      @refFieldsHeader   = ['Record #',
                            'Workbook',           'Workbook Dir',
                            'Data Source Name (tech)', 'Data Source Caption', 'Data Source Name',
                            'Field Name (tech)',  'Field Caption',       'Field Name',
                            'Referenced Field',
                            'Referenced DataSource',
                            'Referenced DataSource Location'
                           ]


      @dictionary      = [
                            { :contents   => @calcFieldsContents,
                              :fileType   => @calcFieldsType,
                              :fileHeader => @calcFieldsHeader
                            },
                            { :contents   => @refFieldsContents,
                              :fileType   => @calcFieldsType,
                              :fileHeader => @calcFieldsHeader
                            }
                          ]
    end

    # def dictionary
    #   @dictionary
    # end

    # def calcFieldsType
    #   @calcFieldsType
    # end

    # def calcFieldsHeader
    #   @calcFieldsHeader
    # end

    # def refFieldsType
    #   @refFieldsType
    # end

    # def calcFieldsHeader
    #   @refFieldsHeader
    # end

    def processTwb twbFileName
       twb     = File.basename(twbFileName)
      @twb     = Twb::Workbook.new twb
      # --
      @calcFieldsFile = "#{twb}.#{@calcFieldsType}.csv"
      @csvCalcFile    = CSV.open(@calcFieldsFile,'w')
      @csvCalcFile << @calcFieldsHeader
      @calcFieldsCount   = 0
      @calcFieldsRecords = []
      # --
      @refFieldsFile = "#{twb}.#{@refFieldsType}.csv" # twb + '.CalcReferenceFields.csv'
      @csvRefFile    = CSV.open(@refFieldsFile,'w')
      @csvRefFile << @refFieldsHeader
      @refFieldsCount   = 0
      @refFieldsRecords = []
      # --
      @csvFiles = [
                    { :contents => @calcFieldsContents,
                      :name     => @calcFieldsFile,
                      :records  => @calcFieldsRecords
                    },
                    { :contents => @refFieldsContents,
                      :name     => @refFieldsFile,
                      :records  => @refFieldsRecords
                    }
                  ]
      # --
      dss = @twb.datasources
      dss.each do |ds|
        calcFields = ds.calculatedFieldsMap.sort_by { |fldName,calc| fldName }
        calcFields.each do |fldname, field|
          formRecord  =  [ @calcFieldsCount += 1,
                           @twb.name,  @twb.dir,
                           ds.name,    ds.caption,    ds.uiname,
                           field.name, field.caption, field.uiname,
                           "#{ds.name}.#{field.name}",
                           field.calculation.formulaFlatResolved,
                           field.calculation.formula.length,
                           field.calculation.is_tableCalc,
                           field.calculation.is_lod,
                           field.calculation.lodCodePos
                         ]
          @calcFieldsRecords.push formRecord
          @csvCalcFile  <<  formRecord
          #--
          field.calculation.calcFields.each do |cf|
            refRecord = [ @refFieldsCount += 1,
                          @twb.name,  @twb.dir,
                          ds.name,    ds.caption,    ds.uiname,
                          field.name, field.caption, field.uiname,
                          cf.uiName,
                          cf.dataSource,  
                          cf.dataSourceRef,
                        ]
            @refFieldsRecords.push refRecord
            @csvRefFile << refRecord               
          end
        end
      end
      @csvCalcFile.close unless @csvCalcFile.nil?
      @csvRefFile.close unless @csvRefFile.nil?
    end

  end # class CSVEmitter

end   # nodule CalculatedFields
end   # module Analysis
end   # module Twb