#  calculatedfieldsanalyzer.rb - this Ruby script Copyright 2017, 2018 Christopher Gerrard
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

require 'kramdown'

module Twb
module Analysis
module CalculatedFields

  class MarkdownEmitter
    include TabTool

    attr_reader :docFileName

    def initialize(**args)
      @args = args
      init
      @funcdoc = {:class=>self.class, :blurb=>'Create  Markdown files (one per Workbook) documenting Calculated Fields', :description=>'Analyze Calculated Fields - create  Markdown files, one per Workbook' }
      @metrics = {}
    end

    def processTWB twb
      # twb = File.basename(twb)
      @twb = twb #Twb::Workbook.new twb
      @docFileName = './ttdoc/' + @twb.name + '.CalculatedFields.md'
      @docFile     = File.open(@docFileName,'w')
      addDocFile @docFile, @docFileName, "Markdown file of Calculated fields for Workbook '#{@twb.name}'"
      @docFile.puts "# #{twb.name}"
      dsNames = @twb.datasourceUINames
      @docFile.puts "#{dsNames.length} Data Sources"
      @twb.datasources.each do |ds|
        @docFile.puts "## #{ds.uiname}"
        @docFile.puts "__has #{ds.calculatedFields.length} calculated fields__\n "
        cnt = 0
        ds.calculatedFields.each do |cf|
          cnt += 1
          @docFile.puts "### '#{cf.uiname}'"
          @docFile.puts "```"
          cf.formulaResolvedLines.each do |l|
            @docFile.puts "#{l.gsub('<<','[').gsub('>>',']')}" 
          end
          @docFile.puts "```"
          if cf.referencedFields.length > 0
            fieldsRefOrder  = []
            fieldsSortSet   = SortedSet.new
            cf.referencedFields.each do |field|
              fieldsRefOrder.push field.uiname
              fieldsSortSet << field.uiname
            end
            if fieldsRefOrder != fieldsSortSet.to_a
              @docFile.puts "```"
              @docFile.puts "Fields - reference order:"
              fieldsRefOrder.each do |field|
                @docFile.puts "    '#{field}'"
              end
              @docFile.puts "```"
            end
            if fieldsSortSet.length > 0
              @docFile.puts "```"
              @docFile.puts "Fields:"
              fieldsSortSet.each do |field|
                @docFile.puts "    '#{field}'"
              end
              @docFile.puts "```"
            end
          end
            @docFile.puts "\n "
        end
        @docFile.puts "counted #{cnt} calculated fields\n "
      end  # twb.datasources.each
      if kramdownAvailable
        emit 'processing Markdown: kramdownAvailable'
      end
      finis
    end    # def processTwb twb

    private

    def kramdownAvailable
      gem_name, *gem_ver_reqs = 'kramdown', '~> 1.17.0'
      gdep = Gem::Dependency.new(gem_name, *gem_ver_reqs)
      # find latest that satisifies
      found_gspec = gdep.matching_specs.max_by(&:version)
      # instead of using Gem::Dependency, you can also do:
      # Gem::Specification.find_all_by_name(gem_name, *gem_ver_reqs)
          # if found_gspec
          #   puts "Requirement '#{gdep}' already satisfied by #{found_gspec.name}-#{found_gspec.version}"
          # else
          #   puts "Requirement '#{gdep}' not satisfied; could be installing..."
          #   # reqs_string will be in the format: "> 1.0, < 1.2"
          #   # reqs_string = gdep.requirements_list.join(', ')
          #   # multi-arg is safer, to avoid injection attacks
          #   # system('gem', 'install', gem_name, '-v', reqs_string)
          # end
    end

  end # class MarkdownEmitter

end   # nodule CalculatedFields
end   # module Analysis
end   # module Twb