#  fieldsaliasesanalyzer.rb - this Ruby script Copyright 2017, 2018 Christopher Gerrard
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

require 'csv'

module Twb
module Analysis

  class FieldsAliasesAnalyzer

      include TabTool

      attr_reader   :calculatedFieldsCount, :referencedFieldsCount, :metrics

      @@csvFileName   = 'FieldAliases.csv'
      @@csvFileHeader = [
                          'Record #',
                          "Record Number",
                          "Workbook",
                          "Data Source",
                          'Data Source Caption',
                          'Data Source (tech)',
                          'Field',
                          'Field Caption',
                          'Field Name (tech)',
                          'Data Type',
                          'Role',
                          'Type',
                          'Field Value',
                          'Field Alias'
                       ]


      def initialize(**args)
          emit "initialize FieldsAliasesAnalyzer args #{args}"
          @args      = args
          @recordDir = !@args.nil? && @args[:recordDir] == true
          @ttdocdir  = @args[:ttdocdir]
          @csvAdd    = @args[:csvMode] == :add
          @csvMode   = @csvAdd ? 'a' : 'w'
          init
          @funcdoc = {:class=>self.class, :blurb=>"Analyze Fields' Aliases", :description=>'Understanding the aliases for field values can be important and useful.',}
          #-- CSV records collectors
          # @csvFormulaFields    = Set.new
          # @csvFormulaLines     = Set.new
          #-- Counters setup --
          # @twbCount              = 0
          @dataSourcesCount   = 0
          @aliasedFieldsCount = 0
          @aliasessCount      = 0
          #--
          # @referencedFields     = SortedSet.new
          #--
          twbdirLabel = @recordDir.nil? ? nil : 'Workbook Dir'
          @csvFile   = initCSV(@@calcFieldsCSVFileName, 'Calculated fields and their formulas.',                      @@csvFileHeader )
          #--
          @localEmit  = false
          # @imageFiles = Array.new
          #--
          # @doGraph = config(:dograph)
      end

      def processTWB twb
        twb.datasources.each do |ds|
          processDS ds
        end
      end

      def processDS ds
        @dsUIName  = ds.uiname
        @dsCaption = ds.caption
        @dsName    = ds.name
        #--
        aliasesNodes = ds.node.xpath('.//aliases')
        aliasesNodes.each do |an|
          processAliases an
        end
      end

      def processAliases aliasesNode
        #-- Field       --
        caption         = d.xpath('../../@caption').text
        techName        = d.xpath('../../@name').text
        name            = if caption == '' then getName(techName) else caption end
        dataType        = d.xpath('../../@datatype').text
        role            = d.xpath('../../@role').text
        type            = d.xpath('../../@type').text

        #-- Alias       --
        aliasKey        = d.xpath('./@key').text
        aliasValue      = d.xpath('./@value').text






  end  # class FieldsAliasesAnalyzer
