#  sheetfieldsanalyzer.rb Copyright (C) 2018  Chris Gerrard
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.  
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

require 'twb'
require 'csv'

module Twb
module Analysis

  class GroupFieldsAnalyzer

    include TabTool

    attr_accessor :localEmit

    def initialize
      init
      @funcdoc     = {:class=>self.class, :blurb=>'Analyze Group Fields.', :description=>'Identifies the Groups and their Members for grouped fields.',}
      #--
      docFileName  = docFile('GroupFields.csv')
      @csv         = CSV.open(docFileName,'w')
      @csv        << ['Workbook','Data Source','Field','Group','Member']
      addDocFile @csv, docFileName, "Workbooks, Data Sources, and their Grouped Fields"
      #--
      @twbCount    = 0
      @dsCount     = 0
      @gfCount     = 0
      @gfmCount    = 0
    end

    def metrics
      {
        # '# of Workbooks'           => @twbCount,
        '# of Data Sources'        => @dsCount,
        '# of Group Fields'        => @gfCount,
        '# of Group Field Members' => @gfmCount
      }
    end

    def processTWB twb
       @twb     = twb
       @twbName = @twb.name
       emit "   -- #{@twbName}"
       @twbCount += 1
       initMarkdown
       parseDataSources
       finis
       $mdFile.close unless $mdFile.nil?
    end

    def initMarkdown
      $mdFile = File.open(docFile("#{@twbName}.GroupFields.md"), 'w')
      $mdFile << "# #{@twbName}\n "
    end

    def parseDataSources
      @twb.datasources.each do |ds|
          $mdFile.puts "\n## #{ds.uiname}" 
          @dsCount += 1
          cfs = ds.calculatedFields
          cfs.each do |cf|
              if cf.isGroup
                @gfCount += 1
                $mdFile.puts "###  #{cf.name}"
                cf.groupMembers.each do |lead,values|
                    $mdFile.puts "* #{lead}"
                    values.each do |v|
                      @gfmCount += 1
                        $mdFile.puts "    >  #{v}"
                        @csv << [@twbName, ds.uiname,cf.uiname,lead,v]
                    end
                end
              end
          end
      end
    end

  end #class SheetFieldsAnalyzer

end # module Analysis
end # module Twb
