#  calculatedfieldsanalyzer.rb - this Ruby script Copyright 2017 Christopher Gerrard
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

require 'twb'

module Twb
module Analysis
module DataSources

  class DataSourceTableFieldsCSVEmitter

    attr_reader :csvFileName, :csvRecords
    attr_reader :dsCount,     :tablesCount, :fieldsCount  

    @@csvFileName        = 'DataSourceTableFields.csv'
    @csvFileDescription = 'Contains CSV records, each containing the details for an individual Field contained in a table in the Data Source. Data Sources are identified within their host Workbook. '

    @@csvHeader = ['Record #',
                   'Workbook',           
                   'Workbook Dir',
                   'Data Source',
                   'Data Source (tech)',
                   'Table',
                   'Field',
                   'Field (code)',
                   'Field (table)',
                  ]


    def initialize
      @csvFile = CSV.open(@@csvFileName, 'w')
      @csvFile << @@csvHeader
      @outputs = Set.new
      @outputs << @@csvFileName
      @dsCount      = 0
      @tablesCount  = 0
      @fieldsCount  = 0
      @csvRecords   = Set.new
    end

    def doc
      {
        :filedoc => [
                     { :file        => @@csvFileName,
                       :description => @csvFileDescription,
                       :header      => @@csvHeader 
                     }
                    ],
        :outputs => @outputs
      }
    end

    def self.csvHeader
      @@csvHeader
    end

    def self.csvFileType
      @@csvFileName
    end

    def outputs
      @outputs
    end

    def processTwb twb
      @recNum = 0
      @twb    = twb if twb.instance_of? Twb::Workbook
      @twb    = Twb::Workbook.new(twb) if twb.instance_of? String
      # --
      dss = @twb.datasources
      dss.each do |ds|
        tables = Set.new
        @dsCount += 1
        ds.node.xpath('./connection/cols/map').each do |cnode|
          # puts cnode
          key         = cnode.attribute('key').text
          codename    = key.gsub(/^\[|\]$/,'')
          fielduiname = ds.fieldUIName(codename)
          value       = cnode.attribute('value').text.gsub(/^\[|\]$/,'')
          parts       = value.split('].[')
          # puts "%-30s  %-45s  %s \n " % [fielduiname, value, parts]
          csvRecord = [ @fieldsCount += 1,
                        @twb.name,
                        @twb.dir,
                        ds.uiname,
                        ds.name,
                        parts[0],
                        fielduiname,
                        codename,
                        parts[1] 
                      ]
          tables.add parts[0]
          @csvRecords.add csvRecord
          @csvFile << csvRecord
        end
        @tablesCount += tables.length
      end
    end

  end # class CSVEmitter


end   # module DataSources
end   # module Analysis
end   # module Twb