#  categoricalColorCodingAnalyzer.rb Copyright (C) 2018  Chris Gerrard
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.  
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

require 'twb'
require 'csv'

module Twb
module Analysis

  class CategoricalColorCodingAnalyzer

    include TabTool

    attr_accessor :localEmit

    def initialize
      init
      @funcdoc    = {:class=>self.class, :blurb=>"Analyze Fields' values categorical color coding from Tableau Workbooks.", :description=>nil,}
      #--
      docFileName = docFile('CategoricalColorMappings.csv')
      $csv  = CSV.open(docFileName,'w')
      $csv  << ['Workbook', 'Workbook Dir', 'Data Source', 'Field Code', 'Field Tech', 'Field', 'Value', 'Colour']
      addDocFile docFileName, "Workbooks, Data Sources, Fields, and the Fields' members' categorical color codings."
      #--
      @twbCnt    = 0
      @dscnt     = 0
      @fieldsCnt = 0
    end

    def metrics
      {
        # '# of Workbooks'         => @twbcount,
        '# of data sources'      => @dscnt,
        '# of Worksheet Fields'  => @fieldsCnt
      }
    end

    def processTWB twb
      @twb = twb
      twbName  = twb.name
      twbDir   = twb.dir
      emit "   -- #{twbName}"
      @twbCnt += 1
      @twbDomainsLoaded = false
      # <style>
      #   <style-rule element='mark'>
      #     <encoding attr='color' field='[none:Calculation_267401288043974656:nk]' type='palette'>
      #       <map to='#1f77b4'>
      #         <bucket>&quot;Consistently Meets Expectations&quot;</bucket>
      twb.datasources.each do |ds|
          dsName = ds.uiname
          puts "\t - #{dsName}"
          coloredFields = ds.node.xpath('.//style/style-rule/encoding[@attr="color"]')
          # puts "\t   #{coloredFields.length} \t #{coloredFields.class} \t #{coloredFields.nil?}"
          coloredFields.each do |cf|
              puts "Attributes: #{cf.attributes}"
              fieldCode = cf['field']
              fieldTech = fieldCode.sub(/^\[/,'').sub(/\]$/,'').sub(/^(none|attr|usr):/,'').sub(/:nk$/,'')
              fieldUI   = ds.fieldUIName fieldTech
              # puts "\t   - #{field}"
              maps  = cf.xpath('./map')
              maps.each do |map|
                  puts "MAP\n---\n#{map}\n--"
                  color  = map['to']
                  value  = ''
                  values = map.xpath('.//bucket')
                  unless values.nil?
                    if values.length == 1
                      value = values.first.text.gsub(/^"/,'').gsub(/"$/,'')
                    else
                      values.each do |bv| 
                        value += "::#{bv.text}"
                      end
                    end
                  end
                  # puts "\t     - #{color}  ->  #{value}"
                  $csv << [twbName, twbDir, dsName, fieldCode, fieldTech, fieldUI, value, color]
                  puts "VALUE: #{value}"
              end
          end
      end
    end

  end #class CategoricalColorCodingAnalyzer

end # module Analysis
end # module Twb
