#  datasourcefilesanalyzer.rb - this Ruby script Copyright 2017, 2018 Christopher Gerrard
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

require 'csv'
require 'pry'

module Twb
module Analysis
module DataSources

  class DataSourceFilesAnalyzer

      include TabTool

      attr_reader   :filesCount

      @@csvFileName   = 'DataSourceFiles.csv'
      @@csvFileHeader = [ 'Record #',
                          'Workbook',
                          'Workbook Dir',
                          'Data Source', 
                          'File Name', 
                          'File Dir',
                          'File Exists?',
                          'File Modified',
                          'File Size'
                        ]

      def initialize(**args)
          emit "initialize DataSourceFilesAnalyzer args #{args}"
          @args      = args
          @recordDir = !@args.nil? && @args[:recordDir] == true
          @ttdocdir  = @args[:ttdocdir]
          @csvAdd    = @args[:csvMode] == :add
          @csvMode   = @csvAdd ? 'a' : 'w'
          init
          @funcdoc = {:class=>self.class, :blurb=>"Analyze Data Sources' Files", :description=>'Identifying the files accessed by Data Sources.',}
          @dataSourcesCount   = 0
          @recNum             = 0
          #--
          @csvFile   = initCSV(@@csvFileName, 'Data Sources and the Files they access.',                      @@csvFileHeader )
          #--
          @localEmit  = false
      end

      def processTWB twb
        @twbName = twb.name
        @twbDir  = twb.dir
        twb.datasources.each do |ds|
          processFiles ds
        end
      end

      def processFiles ds
        dirFiles = Hash.new { |d,fs| d[fs] = Set.new }
        ds.node.xpath('.//connection[@filename]').each do |fnode|
            twbFileDir  = fnode['directory']
            twbFileName = fnode['filename']
            fqFileName  = twbFileDir.nil? ?  twbFileName : twbFileDir + '/' + twbFileName
            fileName = File.basename fqFileName
            fileDir  = File.dirname  fqFileName
            exists   = File.exist?   fqFileName
            modtime  = exists ? File.mtime(fqFileName) : nil
            size     = exists ? File.size(fqFileName)  : nil
            data = [@recNum+=1, @twbName, @twbDir, ds.uiname, fileName, fileDir, exists, modtime, size]
            @csvFile  << data
        end
      end

  end  # class DataSourceFilesAnalyzer

end # module DataSources
end # module Analysis
end # module Twb