#  DataSourceOriginsAnalyzer.rb Copyright (C) 2018  Chris Gerrard
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.  
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

require 'twb'
require 'csv'

module Twb
module Analysis

  class DataSourceOriginsAnalyzer

    include TabTool

    @@recNum = 0

    attr_accessor :localEmit

    def initialize(**args)
      @args = args
      #-- TODO move @csvAdd * #csvMode resolution to TabTool
      @csvAdd  = args[:csvMode] == :add
      @csvMode = @csvAdd ? 'a' : 'w'
      # emit "@csvAdd : #{@csvAdd}" 
      # emit "@csvMode: #{@csvMode}" 
      #--
      init
      @funcdoc    = {:class=>self.class, :blurb=>'Analyze Data Source Origins, i.e. where the data comes from', :description=>nil,}
      #--
      docFileName = docFile('DataSourceOrigins.csv')
      @csvFile  = CSV.open(docFileName,'w')
      unless @csvAdd
        @csvFile  << ['Workbook', 'Workbook Version', 'Workbook Directory', 'Data Source', 'Data Source (tech)', 'Is Published?', "Rec #"]
      end
      addDocFile @csvFile, docFileName, "Workbooks, Data Sources, and the Data Sources' origins"
      #--
      @twbCnt     = 0
      @dsCnt      = 0
    end

    def metrics
      {
        # '# of Workbooks'    => @twbcount,
        '# of Data Sources' => @dsCnt,
      }
    end

    def processTWB twb
       @twb = twb
       emit "   -- twb:: #{@twb.name}"
       @twbCnt += 1
       @twbDomainsLoaded = false
       parseDataSources
       finis
    end

    private

    def parseDataSources
      @twb.datasources.each do |ds|
        @csvFile << [@twb.name, @twb.version, @twb.dir, ds.uiname, ds.name, ds.isPublished, @@recNum+=1]
        @dsCnt += 1
      end
    end

  end #class SheetFieldsAnalyzer

end # module Analysis
end # module Twb
