#  WorksheetFields.rb Copyright (C) 2017, 2018  Chris Gerrard
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.  
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

require 'twb'
require 'csv'

module Twb
module Analysis
module DataSources


  class DataSourcesEnumerator

    include TabTool

    attr_accessor :localEmit

    def initialize(**args)
      @args      = args
      @recordDir = !@args.nil? && @args[:recordDir] == true
      @ttdocdir  = @args[:ttdocdir]
      @csvAdd    = args[:csvMode] == :add
      @csvMode   = @csvAdd ? 'a' : 'w'
      init
      @funcdoc   = {:class=>self.class, :blurb=>"Enumerate Workbooks' Data Sources", :description=>'Documents Data Sources, recoring their names.',}
      #--
      docFileName      = docFile('DataSources.csv')
      @sheetFieldsCSV  = CSV.open( docFileName,@csvMode)
      unless @csvAdd
        if @recordDir
          @sheetFieldsCSV  << ['Rec #','Workbook','Data Source','Data Source (tech)','Class','Extract?','# Connections','# Fields','Published?', '# Tables', '# Aliases', '# Groups', 'Workbook Dir']
        else
          @sheetFieldsCSV  << ['Rec #','Workbook','Data Source','Data Source (tech)','Class','Extract?','# Connections','# Fields','Published?', '# Tables', '# Aliases', '# Groups']
        end
      end
      addDocFile @sheetFieldsCSV, docFileName, "Workbooks and the Data Sources they contain."
      #--
      @dsCount  = 0
      @recNum   = 0
    end

    def metrics
      {
        '# of Data Sources'        => @dsCount,
      }
    end

    def processTWB twb
       @twb     = twb
       @twbName = @twb.name
       @twbDir  = @twb.dir
       emit "   -- #{@twbName}"
       parseDataSources
    end

    def parseDataSources
      @dataSources  = @twb.datasources
      @dataSources.each do |ds|
        @dsCount += 1
        recordCSV [ @twbName, 
                    ds.uiname, 
                    ds.name, 
                    ds.dsclass, 
                    ds.isExtract, 
                    ds.connections.length,
                    ds.allFields.length, 
                    ds.isPublished, 
                    ds.tables.length, 
                    ds.aliases.length, 
                    ds.groups.length 
                  ]
        # [@twbName, @modTime, sheet.name, filter.type, filter.inexclude, filter.dataSource.uiname, filter.uiname, nil,   nil,    nil,  filter.inexMode, filter.includeNull, filter.kind]
      end
    end

    private

      def recordCSV record
        numberedRec = [@recNum+=1] + record
        if @recordDir
          @sheetFieldsCSV  << numberedRec.push(@twbDir)
        else
          @sheetFieldsCSV  << numberedRec
        end
      end

end # class DataSourcesEnumeratorA

end # module DataSources
end # module Analysis
end # module Twb
