#  worksheetsummarizer.rb Copyright (C) 2018  Chris Gerrard
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.  
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

require 'twb'
require 'csv'

module Twb
module Analysis
module DataSources

  class ExtractsAnalyzer

    include TabTool

    attr_accessor :localEmit

    def initialize(**args)
      @args      = args
      @recordDir = !@args.nil? && @args[:recordDir] == true
      @ttdocdir  = @args[:ttdocdir]
      @csvAdd    = !@args.nil? && args[:csvMode] == :add
      @csvMode   = @csvAdd ? 'a' : 'w'
      init
      @funcdoc   = {:class=>self.class, :blurb=>'Analyze Extracts', :description=>'Identifies Data Source Extracts & records relevant data.',}
      #--
      docFileName  = docFile('Extracts.csv')
      @extractsCSV = CSV.open(docFileName,@csvMode)
      #--
        # <extract count='-1' enabled='true' units='records'>
        #   <connection access_mode='readonly' 
        #               authentication='auth-none' 
        #               author-locale='en_US' 
        #               class='hyper' 
        #               dbname='C:/tech/Tableau/Tableau Tools/Ruby/gems/twb/lib/twb/analysis/datasources/data/Sample - World Bank Indicators (Excel).hyper' 
        #               default-settings='yes' 
        #               schema='Extract' 
        #               sslmode='' 
        #               update-time='05/17/2019 10:39:21 PM' 
        #               username='tableau_internal_user'>
        #     <relation name='Extract' table='[Extract].[Extract]' type='table' />
        #     <refresh>
        #       <refresh-event add-from-file-path='Sample - World Bank Indicators (Excel)' increment-value='%null%' refresh-type='create' rows-inserted='2354' timestamp-start='2019-05-17 22:39:21.474' />
        #     </refresh>
        #   </connection>
        # </extract>
        #--
      unless @csvAdd
        @csvHeader = [ 'Rec #', 
                       'Workbook', 
                       'Data Source',
                       'Enabled',
                       'Units',
                       'Access Mode',
                       'Authentication',
                       'Class',
                       'Extract Name (FQ)',
                       'Extract Name',
                       'Extract Directory',
                       'Default Settings',
                       'Schema',
                       'SSL Mode',
                       'Updated (UTC)',
                       'User Name'
                     ]
        if @recordDir
          @csvHeader.push 'Workbook Dir'
        end
        @extractsCSV << @csvHeader
      end
      addDocFile @extractsCSV, docFileName, "Workbooks and their Data Source Extracts."
      #--
      @twbCount     = 0
      @dsCount      = 0
      @extractCount = 0
      @recNum       = 0
    end

    def metrics
      {
        '# of Data Sources' => @dsCount,
        '# of Extracts'     => @extractCount
      }
    end

    def processTWB twb
       @twb     = twb
       @twbName = @twb.name
       @twbDir  = @twb.dir
       emit "   -- #{@twbName}"
       @twbCount += 1
       parseDataSources
       finis
    end

    def parseDataSources
      @dataSources = @twb.datasources
      @dataSources.each do |ds|
        @dsCount += 1
        emit "DATA SOURCE:: #{ds.name}"
        dsNode = ds.node
        exnode = ds.node.at_xpath('./extract')
        unless exnode.nil?
          cnNode = exnode.at_xpath('./connection')
          # emit true, cnNode
          unless cnNode.nil?
            @extractCount += 1
            dbName = cnNode['dbname']
            # emit true, "cnNode['dbname'] :: #{cnNode['dbname']}"
            # unless dbName.nil?
              exName = File.basename dbName
              exDir  = File.dirname  dbName
            # end
            recordCSV [ @twbName,
                        ds.uiname,
                        exnode['enabled'],
                        exnode['units'],
                        cnNode['access_mode'],
                        cnNode['authentication'],
                        cnNode['class'],
                        dbName,
                        exName,
                        exDir,
                        cnNode['default-settings'],
                        cnNode['schema'],
                        cnNode['sslmode'],
                        cnNode['update-time'],
                        cnNode['username']
                      ]
          end
        end
      end
    end

        # <extract count='-1' enabled='true' units='records'>
        #   <connection access_mode='readonly' 
        #               authentication='auth-none' 
        #               author-locale='en_US' 
        #               class='hyper' 
        #               dbname='C:/tech/Tableau/Tableau Tools/Ruby/gems/twb/lib/twb/analysis/datasources/data/Sample - World Bank Indicators (Excel).hyper' 
        #               default-settings='yes' 
        #               schema='Extract' 
        #               sslmode='' 
        #               update-time='05/17/2019 10:39:21 PM' 
        #               username='tableau_internal_user'>
        #     <relation name='Extract' table='[Extract].[Extract]' type='table' />
        #     <refresh>
        #       <refresh-event add-from-file-path='Sample - World Bank Indicators (Excel)' increment-value='%null%' refresh-type='create' rows-inserted='2354' timestamp-start='2019-05-17 22:39:21.474' />
        #     </refresh>
        #   </connection>
        # </extract>
        #--
      # unless @csvAdd
      #   @csvHeader = [ 'Rec #', 
      #                  'Workbook', 
      #                  'Data Source',
      #                  'Enabled',
      #                  'Units',
      #                  'Access Mode',
      #                  'Authentication',
      #                  'Class',
      #                  'Db Name',
      #                  'Default Settings',
      #                  'Schema',
      #                  'SSL Mode',
      #                  'Updated',
      #                  'User Name'
      #                ]




    private

      def recordCSV record
        numberedRec = [@recNum+=1] + record
        if @recordDir
          @extractsCSV  << numberedRec.push(@twbDir)
        else
          @extractsCSV  << numberedRec
        end
      end

  end # class ExtractsAnalyzer

end # module DataSources
end # module Analysis
end # module Twb
