#  fieldsaliasesanalyzer.rb - this Ruby script Copyright 2017, 2018 Christopher Gerrard
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

require 'csv'

module Twb
module Analysis
module DataSources

  class FieldsAliasesAnalyzer

      include TabTool

      attr_reader   :calculatedFieldsCount, :referencedFieldsCount, :metrics

      @@csvFileName   = 'FieldAliases.csv'
      @@csvFileHeader = [
                          'Record #',
                          "Workbook",
                          "Data Source",
                          'Field',
                          'Value - Db',
                          'Value - Alias'
                        ]


      def initialize(**args)
          emit "initialize FieldsAliasesAnalyzer args #{args}"
          @args      = args
          @recordDir = !@args.nil? && @args[:recordDir] == true
          @ttdocdir  = @args[:ttdocdir]
          @csvAdd    = @args[:csvMode] == :add
          @csvMode   = @csvAdd ? 'a' : 'w'
          init
          @funcdoc = {:class=>self.class, :blurb=>"Analyze Fields' Aliases", :description=>'Understanding the aliases for field values can be important and useful.',}
          #-- CSV records collectors
          # @csvFormulaFields    = Set.new
          # @csvFormulaLines     = Set.new
          #-- Counters setup --
          # @twbCount              = 0
          @dataSourcesCount   = 0
          @aliasedFieldsCount = 0
          @aliasessCount      = 0
          @recNum             = 0
          #--
          # @referencedFields     = SortedSet.new
          #--
          @csvFile   = initCSV(@@csvFileName, 'Fields and their aliased values.',                      @@csvFileHeader )
          #--
          @localEmit  = false
          # @imageFiles = Array.new
          #--
          # @doGraph = config(:dograph)
      end

      def processTWB twb
        @twbName = twb.name
        @twDir   = twb.dir
        twb.datasources.each do |ds|
          processDS ds
        end
      end

      def processDS ds
        @dsUIName  = ds.uiname
        @dsCaption = ds.caption
        @dsName    = ds.name
        #--
        ds.columnFields.each do |fld|
          if fld.hasaliases
            @fieldName    = fld.name
            @fieldCaption = fld.caption
            @fieldUIName  = fld.uiname
            processAliases(fld) 
          end
        end
      end

      def processAliases field
        aliasNodes = field.node.xpath('./aliases//alias')
        aliasNodes.each do |anode|
          key      = anode['key']
          value    = anode['value']
          numberedRec = [
                          @recNum +=1, 
                          @twbName,
                          @dsUIName,
                          @fieldName,
                          anode['key'].to_s.gsub(/^["]+/,'').gsub(/["]+$/,''),
                          anode['value']
                        ]
          #--
          if @recordDir
            @csvFile  << numberedRec.push(@twbDir)
          else
            @csvFile  << numberedRec
          end
        end
      end

  end  # class FieldsAliasesAnalyzer

end # module DataSources
end # module Analysis
end # module Twb