#  sheetfieldsanalyzer.rb Copyright (C) 2018  Chris Gerrard
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.  
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

require 'twb'
require 'csv'
# require 'set'

module Twb
module Analysis

  class ParametersAnalyzer

    include TabTool

    attr_accessor :localEmit
    attr_reader   :twbname, :twbcount

    def initialize
      init
      #-- set up metrics
      @paramscount = 0
      #--
      @funcdoc    = {:class=>self.class, :blurb=>'Analyzing Parameters from Tableau Workbooks.', :description=>nil,}
      docFileName = docFile('Parameters.csv')
      @csv        = CSV.open(docFileName, 'w')
      @csv       << ["Workbook",'Parameter','Caption','Name (tech)','Type','Format','Type (custom)','Type (domain)','Role', 'Value (current)', 'Value','Value (tech)']
                  # [ @twbname,  name,       caption,  nameTech,     type,  format,  typeCustom,     domainType,     role]
      @docfiles = [{:name=>docFileName,:description=>"CSV File containing the data relevant for Parameters."}]
    end

    #-- Parameter attributes
    # caption                    : 11
    # name                       : 11
    # datatype                   : 11
    # datatype-customized        : 9

    # param-domain-type          : 11
    # role                       : 11
    # type                       : 11
    # value                      : 11
    # default-format             : 7
    # alias                      : 1

    def processTWB twb
      if Twb::Workbook != twb.class
        @twb = Twb::Workbook.new twb
      else
        @twb = twb
      end
      @twbname = @twb.name
      emit  "Workbook:: #{@twbname}"
      paramsDS = @twb.datasource 'Parameters'
      @twb.parameters.each do |pName,param|
        @paramscount += 1
        param.values.each do |values|
          @csv       << [ @twbname,
                          param.uiname,
                          param.caption,
                          param.name,
                          param.type,
                          param.format,
                          param.dataTypeCustom,
                          param.domainType,
                          param.role,
                          param.currentValue, # 'Value (current)', 
                          values[:value],     # 'Value',
                          values[:valueTech]      # 'Value (tech)'
                        ]
        end
      end
    end

    def metrics
      {
        '# of Parameters' => @paramscount,
      }
    end

  end #class ParametersAnalyzer

end # module Analysis

end # module Twb
