#  sheetfieldsanalyzer.rb Copyright (C) 2018  Chris Gerrard
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.  
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

require 'twb'
require 'csv'

module Twb
module Analysis

  class DashboardsSummarizer

    include TabTool

    attr_accessor :localEmit

    def initialize(**args)
      @args      = args
      @recordDir = !@args.nil? && @args[:recordDir] == true
      @ttdocdir  = @args[:ttdocdir]
      @csvAdd    = !@args.nil? && args[:csvMode] == :add
      @csvMode   = @csvAdd ? 'a' : 'w'
      init
      @funcdoc   = {:class=>self.class, :blurb=>'Analyze Dashboard Worksheets', :description=>'Identifies the Worksheets present in Dashboards.',}
      #--
      docFileName     = docFile('DashboardSummaries.csv')
      @dashboardsCSV  = CSV.open(docFileName,@csvMode)
      unless @csvAdd
        if @recordDir
          @dashboardsCSV  << ['Rec #','Workbook','Dashboard','# Worksheets','Workbook Dir']
        else
          @dashboardsCSV  << ['Rec #','Workbook','Dashboard','# Worksheets'               ]
        end
      end
      addDocFile @dashboardsCSV, docFileName, "Workbooks and their Dashboards' summaries"
      #--
      @twbCount    = 0
      @dashCount   = 0
      @recNum      = 0
    end

    def metrics
      {
        # '# of Workbooks'  => @twbCount,
        '# of Dashboards' => @dashCount,
      }
    end

    def processTWB twb
       @twb     = twb
       @twbName = @twb.name
       @twbDir  = @twb.dir
       @modTime = @twb.modtime
       emit "   -- #{@twbName}"
       @twbCount += 1
       parseDashes
       finis
    end

    def parseDashes
      @dashboards = @twb.dashboards
      @dashboards.each do |dash|
        emit "DASH:: #{dash.name}"
        @dashCount += 1
        recordCSV [@twbName, dash.name, dash.worksheets.length]
      end
    end

    private

      def recordCSV record
        numberedRec = [@recNum+=1] + record
        if @recordDir
          @dashboardsCSV  << numberedRec.push(@twbDir)
        else
          @dashboardsCSV  << numberedRec
        end
      end

  end #class SheetFieldsAnalyzer

end # module Analysis
end # module Twb
