#  WorksheetFields.rb Copyright (C) 2017, 2018  Chris Gerrard
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.  
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

require 'twb'
require 'csv'

module Twb
module Analysis

  class SheetFiltersAnalyzer

    include TabTool

    attr_accessor :localEmit

    def initialize(**args)
      @args      = args
      # puts "@args: #{@args}"
      @recordDir = !@args.nil? && @args[:recordDir] == true
      @ttdocdir  = @args[:ttdocdir]
      @csvAdd    = args[:csvMode] == :add
      @csvMode   = @csvAdd ? 'a' : 'w'
      init
      @funcdoc   = {:class=>self.class, :blurb=>'Analyze Worksheet Filters', :description=>'Documents Quick Filters and the values they employ, if any. Work in progess.',}
      #--
      docFileName      = docFile('WorksheetFilters.csv')
      @sheetFieldsCSV  = CSV.open( docFileName,@csvMode)
      unless @csvAdd
        if @recordDir
          @sheetFieldsCSV  << ['Rec #','Workbook','Workbook Modified','Worksheet','Filter Type','Operation','Data Source','Field','Value','Alias', 'Alias?','Operation Mode','Include Null?','Kind','Workbook Dir']
        else
          @sheetFieldsCSV  << ['Rec #','Workbook','Workbook Modified','Worksheet','Filter Type','Operation','Data Source','Field','Value','Alias', 'Alias?','Operation Mode','Include Null?','Kind']
        end
      end
      addDocFile @sheetFieldsCSV, docFileName, "Workbooks, Worksheets and the sheets' Quick Filters"
      #--
      @sheetCount  = 0
      @filterCount = 0
      @recNum      = 0
    end

    def metrics
      {
        '# of Worksheets'        => @sheetCount,
        '# of Worksheet Filters' => @filterCount
      }
    end

    def processTWB twb
       @twb     = twb
       @twbName = @twb.name
       @twbDir  = @twb.dir
       @modTime = @twb.modtime
       emit "   -- #{@twbName}"
       @twbDomainsLoaded = false
       parseFilters
    end

    def parseFilters
      @worksheets  = @twb.worksheets
      @worksheets.each do |sheet|
        emit "\n\nSHEET: #{sheet.name}"
        filters = sheet.filters
        filters.each do |filter|
          filter.emit "-----------------------------\nWORKSHEET:: #{sheet.name}\n-----------------------------"
          if filter.values.empty?
          # @sheetFieldsCSV  << ['Workbook','Worksheet','Filter Type','Operation'     ,'Data Source','Field','Value','Alias', 'Alias?']
            recordCSV [@twbName, @modTime, sheet.name, filter.type, filter.inexclude, filter.dataSource.uiname, filter.uiname, nil,   nil,    nil,  filter.inexMode, filter.includeNull, filter.kind]
          end
          filter.values.each do |valueMap|
            value  = valueMap[:value]
            valias = valueMap[:alias]
            same   = value.eql? valias
            emit "RECORDING FILTER VALUES:  %-35s  --  %-25s  same? %s " % [value,valias,same]
            recordCSV [@twbName, @modTime, sheet.name, filter.type, filter.inexclude, filter.dataSource.uiname, filter.uiname, value, valias, same, filter.inexMode, filter.includeNull, filter.kind]
          end
        end
      end
    end

    private

      def recordCSV record
        numberedRec = [@recNum+=1] + record
        if @recordDir
          @sheetFieldsCSV  << numberedRec.push(@twbDir)
        else
          @sheetFieldsCSV  << numberedRec
        end
      end

end # class SheetFiltersAnalyzerA

end # module Twb
end # module Analysis
