#  WorksheetFields.rb Copyright (C) 2017, 2018  Chris Gerrard
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.  
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

require 'twb'
require 'csv'

module Twb
module Analysis

  class SheetFiltersAnalyzerA

    include TabTool

    attr_accessor :localEmit

    def initialize
      init
      @funcdoc     = {:class=>self.class, :blurb=>'Analyze Worksheet filters.', :description=>'Documents Quick Filters and the values they employ, if any. Work in progess.',}
      #--
      docFileName  = docFile('WorksheetFiltersA.csv')
      $sheetFieldsCSV  = CSV.open( docFileName  ,'w')
      $sheetFieldsCSV  << ['Workbook','Worksheet','Filter Type','Operation','Data Source','Field','Value','Alias', 'Alias?']
      addDocFile docFileName, "Workbooks, Worksheets and the sheets' Quick Filters"
      #--
      @sheetCount  = 0
      @filterCount = 0
    end

    def metrics
      {
        '# of Worksheets'        => @sheetCount,
        '# of Worksheet Filters' => @filterCount
      }
    end

    def processTWB twb
       @twb     = twb
       @twbName = @twb.name
       emit "   -- #{@twbName}"
       @twbDomainsLoaded = false
       parseFilters
    end

    def parseFilters
      @worksheets  = @twb.worksheets
      @worksheets.each do |sheet|
        emit "\n\nSHEET: #{sheet.name}"
        filters = sheet.filters
        filters.each do |filter|
          filter.emit "-----------------------------\nWORKSHEET:: #{sheet.name}\n-----------------------------"
          if filter.values.empty?
          # $sheetFieldsCSV  << ['Workbook','Worksheet','Filter Type','Operation'     ,'Data Source','Field','Value','Alias', 'Alias?']
            $sheetFieldsCSV  << [@twbName  ,sheet.name, filter.type  ,filter.inexclude, filter.dataSource.uiname, filter.uiname,nil,nil,nil]
          end
          filter.values.each do |valueMap|
            value  = valueMap[:value]
            valias = valueMap[:alias]
            same   = value.eql? valias
            # puts "RECORDING FILTER VALUES:  %-25s  --  %-25s  same? %s " % [value,valias,same]
            $sheetFieldsCSV  << [@twbName  ,sheet.name, filter.type  ,filter.inexclude, filter.dataSource.uiname, filter.uiname, value, valias, same]
          end
        end
      end
    end

end # class SheetFiltersAnalyzerA

end # module Twb
end # module Analysis
