#  sheetfieldsanalyzer.rb Copyright (C) 2019  Chris Gerrard
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.  
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

require 'twb'
require 'csv'

module Twb
module Analysis

  class SheetsInTooltipAnalyzer

    include TabTool

    def initialize(**args)
      @args      = args
      @recordDir = !@args.nil? && @args[:recordDir] == true
      @ttdocdir  = @args[:ttdocdir]
      @csvAdd    = !@args.nil? && args[:csvMode] == :add
      @csvMode   = @csvAdd ? 'a' : 'w'
      init
      @funcdoc   = {:class=>self.class, :blurb=>'Tooltip Vizzes', :description=>'Identifies the vizzes used in tooltips.',}
      #--
      docFileName     = docFile('SheetsInTooltips.csv')
      @csvFile  = CSV.open(docFileName,@csvMode)
      unless @csvAdd
        if @recordDir
          @csvFile  << ['Rec #','Workbook','Worksheet - Host','Worksheet - Toolip viz','Max Width','Max Height', 'Workbook Dir']
        else
          @csvFile  << ['Rec #','Workbook','Worksheet - Host','Worksheet - Toolip viz','Max Height','Max Width']
        end
      end
      addDocFile @csvFile, docFileName, "Workbooks, Worksheets with Vizzes in their Tooltips"
      #--
      @twbCount    = 0
      @sheetCount  = 0
      @ttvizCount  = 0
      @recNum      = 0
    end

    def metrics
      {
        # '# of Workbooks'  => @twbCount,
        '# of Worksheets' => @sheetCount,
        '# of Worksheets in Tooltips' => @ttvizCount
      }
    end

    def processTWB twb
       @twb     = twb
       @twbName = @twb.name
       @twbDir  = @twb.dir
       @modTime = @twb.modtime
       emit "   -- #{@twbName}"
       @twbCount += 1
       parseSheets
       finis
    end

    def parseSheets
      @sheets      = @twb.worksheets
      # puts " #Sheets: #{@sheets.length}"
      @sheetCount += @sheets.length
      @sheets.each do |sheet|
        # puts "SHEET:: #{sheet.name}"
        # @dashCount += 1
        ttip = sheet.tooltip
        runs = ttip.xpath('.//run')
        runs.each do |run|
          # puts run.text
          text = run.text
          isViz = text =~ /.*<Sheet[ ]+name=.*>.*/
          # puts "\t   #{text}"
          if isViz
            vizCode = text.gsub(/^[^<]*/,'').gsub(/[^>]*$/,'')
            vizNode = Nokogiri::XML(vizCode).at_xpath('./Sheet')
            # puts "\t   #{vizCode}"
            # puts "\t   #{vizNode.class}  \t #{vizNode}"
            recordCSV [@twbName, sheet.name, vizNode['name'], vizNode['maxwidth'], vizNode['maxheight']]
          end
          
          # @sheetCount += 1
          # recordCSV [@twbName, dash.name, sheet.name, sheet.hidden, sheet.visible ]
        end
      end
    end

    private

      def recordCSV record
        numberedRec = [@recNum+=1] + record
        if @recordDir
          @csvFile  << numberedRec.push(@twbDir)
        else
          @csvFile  << numberedRec
        end
      end


  end #class SheetFieldsAnalyzer

end # module Analysis
end # module Twb
