#  sheetfieldsanalyzer.rb Copyright (C) 2018  Chris Gerrard
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.  
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

require 'twb'
require 'csv'

module Twb
module Analysis

  class SheetSourcesAnalyzer

    include TabTool

    attr_accessor :localEmit

    def initialize
      init
      @funcdoc    = {:class=>self.class, :blurb=>'Analyze Worksheet Fields', :description=>nil,}
      #--
      docFileName = docFile('WorksheetFields.csv')
      @sheetFieldsCSV  = CSV.open(docFileName,'w')
      @sheetFieldsCSV  << ['Workbook','Worksheet','Data Source','Data Source (tech)','Field','Field (tech)','Usage']
      addDocFile docFileName, "Workbooks, Worksheets, and the Sheets' Data Sources and Fields"
      #--
      @twbCnt     = 0
      @sheetCnt   = 0
      @fieldsCnt  = 0
    end

    def metrics
      {
        # '# of Workbooks'         => @twbcount,
        '# of Worksheets'        => @sheetCnt,
        '# of Worksheet Fields'  => @fieldsCnt
      }
    end

    def processTWB twb
       @twb = twb
       emit "   -- twb:: #{@twb.name}"
       @twbCnt += 1
       @twbDomainsLoaded = false
       parseSheets
       finis
    end

    private

    def parseSheets
      @worksheets  = @twb.worksheets
      @worksheets.each do |sheet|
        @sheet     = sheet.name
        @sheetCnt += 1
        emit "SHEET: #{@sheet}"
        showFields sheet unless sheet.datasourceFields.nil?
      end
    end

    def showFields sheet
      showDBFields  sheet
      showRCFields  sheet.rowFields, :row
      showRCFields  sheet.colFields, :column
    end

    def showDBFields sheet
      fields = sheet.datasourceFields
      emit "def showDBFields sheet: #{sheet.name} #FIELDS: #{fields.length}"
      if fields.nil?
        @sheetFieldsCSV  << [@twb.name, @sheet, nil, nil, nil, nil, nil]
      end
      fields.each do |dsName, dsfields|
        ds = @twb.datasource dsName
        emit "      ds: #{dsName}"
        emit "        - #{ds.uiname}"
        emit "        : #{ds.class}"
        dsfields.each do |sheetField|
           @fieldsCnt += 1
           emit "       f: #{sheetField}"
           emit "       c: #{sheetField.class}"
           fuiName = ds.fieldUIName sheetField #Fields[sheetField]
           @sheetFieldsCSV  << [@twb.name, @sheet, ds.uiname, dsName, sheetField.uiname, sheetField.name, 'DB ref']
           # emit true, "        : #{dsFields[field].class}"
        end
      end
    end

    def showRCFields fields, usage
      emit "def showRCFields #fields: #{fields.length}  \t #{fields}"
      if fields.nil?
        @sheetFieldsCSV  << [@twb.name, @sheet, nil, nil, nil, nil, nil]
      else
        fields.each do |cf|
          emit "coded field: #{cf}"
          fldName = cf.name
          dsName  = cf.dataSource
          ds      = @twb.datasource cf.dataSource
          emit "DATASOURCE : #{ds.class} "  #{ }" #{ds}"
          fuiName = ds.fieldUIName cf.name
          emit "      ds: #{dsName}"
          emit "        - #{ds.uiname}"
          emit "        : #{ds.class}"
          @sheetFieldsCSV  << [@twb.name, @sheet, ds.uiname, dsName, fuiName, fldName, usage]
        end
      end
    end

  end #class SheetFieldsAnalyzer

end # module Analysis
end # module Twb
