#  worksheetsummarizer.rb Copyright (C) 2018  Chris Gerrard
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.  
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

require 'twb'
require 'csv'

module Twb
module Analysis

  class WorksheetSummarizer

    include TabTool

    attr_accessor :localEmit

    # Worksheet attributes; from Worksheet.rb
      # attr_reader   :node,   :name,      :datasourcenames, :datasources
      # attr_reader   :panesCount
      # attr_reader   :fields, :rowFields, :colFields, :paneFields, :datasourceFields, :pageFields, :encodedFields, :slicesFields
      # attr_reader   :filters
      # attr_reader   :tooltip
      # attr_accessor :hidden, :visible

    def initialize(**args)
      @args      = args
      @recordDir = !@args.nil? && @args[:recordDir] == true
      @ttdocdir  = @args[:ttdocdir]
      @csvAdd    = !@args.nil? && args[:csvMode] == :add
      @csvMode   = @csvAdd ? 'a' : 'w'
      init
      @funcdoc   = {:class=>self.class, :blurb=>'Summarize Worksheets', :description=>'Identifies Worksheets & summarizes them.',}
      #--
      docFileName     = docFile('WorkheetSummaries.csv')
      @worksheetsCSV  = CSV.open(docFileName,@csvMode)
      unless @csvAdd
        @csvHeader = [ 'Rec #', 
                       'Workbook', 
                       'Worksheet',
                       'Hidden', 'Visible',
                       '# Data Sources', 
                       '# Fields - Data', '# Fields - Rows', '# Fields - Cols',
                       'Tooltip?', 'Filters?'
                     ]
        if @recordDir
          @csvHeader.push 'Workbook Dir'
        end
        @worksheetsCSV << @csvHeader
      end
      addDocFile @worksheetsCSV, docFileName, "Workbooks and their Worksheets' summaries."
      #--
      @twbCount    = 0
      @sheetCount  = 0
      @recNum      = 0
    end

    def metrics
      {
        '# of Worksheets' => @sheetCount
      }
    end

    def processTWB twb
       @twb     = twb
       @twbName = @twb.name
       @twbDir  = @twb.dir
       @modTime = @twb.modtime
       emit "   -- #{@twbName}"
       @twbCount += 1
       parseSheets
       finis
    end

    def parseSheets
      @worksheets  = @twb.worksheets
      @worksheets.each do |sheet|
        emit "SHEET:: #{sheet.name}"
        @sheetCount += 1
        recordCSV [ @twbName, 
                    sheet.name, 
                    sheet.hidden, 
                    sheet.visible,
                    sheet.datasources.length,
                    sheet.datasourceFields.length,
                    sheet.rowFields.length,
                    sheet.colFields.length,
                    !sheet.tooltip.nil?,
                    !sheet.filters.nil?,
                  ]
      end
    end

    private

      def recordCSV record
        numberedRec = [@recNum+=1] + record
        if @recordDir
          @worksheetsCSV  << numberedRec.push(@twbDir)
        else
          @worksheetsCSV  << numberedRec
        end
      end

  end #class WorksheetSummarizer

end # module Analysis
end # module Twb
