#  sheetfieldsanalyzer.rb Copyright (C) 2018  Chris Gerrard
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.  
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

require 'twb'
require 'csv'

module Twb
module Analysis

  class WorkbookSummaryAnalyzer

    include TabTool

    attr_accessor :localEmit

    def initialize(**args)
      @args      = args
      @recordDir = !@args.nil? && @args[:recordDir] == true
      @ttdocdir  = @args[:ttdocdir]
      @csvAdd    = !@args.nil? && args[:csvMode] == :add
      @csvMode   = @csvAdd ? 'a' : 'w'
      init
      @funcdoc   = {:class=>self.class, :blurb=>'Analyze Workbooks', :description=>'Workbook summary data: # Data Sources; Dashboards; Sheets; etc.'}
      #--
      docFileName  = docFile('WorkbookSummary.csv')
      @csvFile     = CSV.open(docFileName,@csvMode)
      unless @csvAdd
        csvHeader       = ['Rec #', 'Workbook','Directory','Type','Version','Build','Platform','Modified','# Data Sources','# Dashboards','# Worksheets']
        if @recordDir
          csvHeader.push('Workbook Directory')
        end
        @csvFile  << csvHeader 
      end
      addDocFile @csvFile, docFileName, "Workbook summary data."
      #--
      @twbCount    = 0
      @dashCount   = 0
      @dsCount     = 0
      @sheetCount  = 0
      @recNum      = 0
    end

    def metrics
      {
        # '# of Workbooks'    => @twbCount,
        '# of Data Sources' => @dsCount,
        '# of Dashboards'   => @dashCount,
        '# of Worksheets'   => @sheetCount
      }
    end

    def processTWB twb
       @twb     = twb
       emit "   -- #{@twbName}"
       recordCSV [ @twb.name,
                   @twb.dir, 
                   @twb.type, 
                   @twb.version, 
                   @twb.build,
                   @twb.platform,
                   # @twb.base, 
                   @twb.modtime, 
                   @twb.datasources.length,
                   @twb.dashboards.length,
                   @twb.worksheets.length
                 ]
       @twbCount   += 1
       @dsCount    += @twb.datasources.length
       @dashCount  += @twb.dashboards.length
       @sheetCount += @twb.worksheets.length
       finis
    end

    private

      def recordCSV record
        numberedRec = [@recNum+=1] + record
        if @recordDir
          @csvFile  << numberedRec.push(@twb.dir)
        else
          @csvFile  << numberedRec
        end
      end

  end #class SheetFieldsAnalyzer

end # module Analysis
end # module Twb
