#  Copyright (C) 2014, 2015, 2020 Chris Gerrard
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

require 'nokogiri'
require 'digest/md5'

module Twb

  class CalculatedField

    include TabTool

    attr_reader :dataSource
    attr_reader :node,        :properties
    attr_reader :caption,     :name,       :uiname
    attr_reader :datatype,    :role,       :propType  
    attr_reader :calculation, :referencedFields 
    attr_reader :isGroup,     :groupMembers
    attr_reader :hidden

    def initialize(calcNode, datasource=nil)
        @node       = calcNode
        @dataSource = datasource
        # -- 
        @caption    = calcNode.attribute('caption').text if calcNode.has_attribute?('caption')
        @name       = calcNode.attribute('name').text.gsub(/^\[/,'').gsub(/\]$/,'')
        @uiname     = caption.nil? ? @name : @caption
        # -- 
        @datatype    = @node.attribute('datatype').text
        @role        = @node.attribute('role').text
        @propType    = @node.attribute('type').text   # n.b. 'type' is used as a proxy for class
        # -- 
        @calculation = Twb::FieldCalculation.new(self, datasource)
        # -- 
        @hidden      = true if calcNode.has_attribute?('caption')
        @isGroup     = !@node.at_xpath('./calculation[@class="categorical-bin"]').nil?
    end

    def properties
      @properties ||= loadProperties
    end

    def referencedFields
        @calculation.referencedFields
    end

    def formulaLines
        @calculation.formulaLines
    end

    def formulaResolvedLines
        @calculation.formulaResolvedLines
    end

    def formulaResolved
        @calculation.formulaResolved
    end

    def groupMembers
        @groupMembers ||= loadgroupmembers
    end

    def binValues binName
        values   = []
        binValue = "\"#{binName}\""
        binNode = @node.at_xpath("./calculation[@class='categorical-bin']/bin[@value='#{binValue}']")
        unless binNode.nil?
            binNode.xpath('.//value').each do |v|
                values << v.text.gsub(/^['"]|['"]$/,'')
            end
        end
        return values
    end

    def loadProperties
      @properties= {}
      @node.attributes.each do |name,attr|
        @properties[name] = attr.value
      end
      @properties[:uiname] = @uiname
      @properties[:uuid]   = uuid
      return @properties
    end

    def id
        @id ||= @id = "#{@dataSource.uiname}::#{@uiname}"
    end

    # def uuid
    #     @uuid ||= loadUUID
    # end

    # def loadUUID
    #     dsn = @dataSource.nil? ? 'NO DATASOURCE' : dataSource.uuid 
    #     @uuid = Digest::MD5.hexdigest("#{dsn}::@name").hash
    # end

    def to_s
        "%s(%s) => %s" % [uiname, name, @calculation.formulaFlat]
    end

    private

    def loadgroupmembers
        @groupMembers = Hash.new { |h,k| h[k] = [] } 
        if @isGroup
            groupLeads =  @node.xpath('./calculation[@class="categorical-bin"]//bin')
            groupLeads.each do |gl|
                lead = gl['value']
                groupAlias = @node.at_xpath("./aliases/alias[@key='#{lead}']")
                groupNamex = groupAlias.nil? ? lead : groupAlias['value']
                groupName  = groupNamex.gsub(/^['"]|['"]$/,'')
                values = gl.xpath('./value')
                values.each do |v|
                    @groupMembers[groupName] << v.text.gsub(/^['"]|['"]$/,'').gsub('\"','"')
                end
            end
        end
        return @groupMembers
    end

  end # class
end # module
