#  Copyright (C) 2014, 2015, 2017  Chris Gerrard
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

require 'nokogiri'
require 'digest/md5'
require 'csv'

module Twb

  class CodedField

    include Comparable

    attr_reader :dataSource
    attr_reader :name, :techCode, :function, :rawCode 

    def initialize code
      # puts "...CodedField :: #{code}"
      @rawCode = code
      trimmed  = code.gsub(/^['"]|['"]$/,'').gsub(/^\[|\]$/,'').gsub(/:[0-9]+$/,'')
      # puts "      trimmed :: #{trimmed}"
      parts    = trimmed.split('].[')
     #puts "Field: #{code}  parts: #{parts.length} - #{parts.inspect}"
     #puts "   p1: #{parts[0]}"
     #puts "   p2: #{parts[1]}"
      if parts.length == 1
       #puts '==1'
        @name = parts[0]
        @techCode = "[#{@name}]"
      elsif # parts.length <> 1
       #puts '<>1'
       #puts "p[0]: #{parts[0]}"
       #puts "p[1]: #{parts[1]}"
        @dataSource  = parts[0]
        fldName = parts[1]
        if fldName.start_with?(':') && fldName.count(':') == 1
          @name = fldName
        else
          parseField fldName
        end
        @techCode   = "[#{@dataSource}].[#{@name}]"
      end
    end # initialize

    def parseField str
      # puts "parseField: #{str}"
      parts = str.split(':')
      case parts.length
      when 1
        @name     = parts[0]
      else
        @name     = parts[-2]
        @function = parts[0]
      end
    end

    def id
      @id ||= @id = "#{@dataSource}::#{@uiname}"
    end

    def to_s
      "ds:#{dataSource}|n:#{name}|tc:#{techCode}"
    end

    def <=>(other)
      # myName    = @uiname.nil? ? '' : @uiname
      # otherName = other.uiName.nil? ? "" : other.uiName
      # ##puts "#{@uiname} / #{myName} <=>  #{otherName} / #{other.uiName}"
      # ##puts "#{@uiname.nil?} // #{other.uiName.nil?}"
      # myName <=> otherName
      @fqName <=> other.techCode
    end

  end # class CodedField

end  # module Twb
