#  Copyright (C) 2014, 2015, 2016  Chris Gerrard
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

# require 'nokogiri'
# require 'digest/md5'
require 'json'

module Twb

  class DataSource

    include TabTool
  
    @@hasher = Digest::SHA256.new

    @@connGNodeParamsJSON = %q(
    { "csv"           : { "label" : ["filename"], "id" : ["directory","filename"], "type" : "CSV"        },
      "excel"         : { "label" : ["filename"], "id" : ["directory","filename"], "type" : "Excel"      },
      "dataengine"    : { "label" : ["dbname"  ], "id" : ["directory","filename"], "type" : "TDE"        },
      "msaccess"      : { "label" : ["filename"], "id" : ["directory","filename"], "type" : "MS Access"  },
      "oracle"        : { "label" : ["server"  ], "id" : [            "server"  ], "type" : "Oracle"     },
      "postgres"      : { "label" : ["server"  ], "id" : [            "server"  ], "type" : "PostgreSQL" },
      "textscan"      : { "label" : ["filename"], "id" : ["directory","filename"], "type" : "CSV / TSV"  },
      "excel-direct"  : { "label" : ["filename"], "id" : [            "filename"], "type" : "Excel"      },
      "salesforce"    : { "label" : ["server"  ], "id" : [            "server"  ], "type" : "Salesforce" }
    }
    )
    @@cgNodeParams  = JSON.parse @@connGNodeParamsJSON


    attr_reader :workbook
    attr_reader :name,          :caption,         :uiname
    attr_reader :attributes
    attr_reader :dsclass,       :isExtract
    attr_reader :connection,    :connections,     :connHash,   :connAttributes       
    attr_reader :isPublished
    attr_reader :uuid
    attr_reader :tables,        :joinPairs      
    attr_reader :localFields,   :localFieldNames, :localField, :hasField 
    attr_reader :columnFields 
    attr_reader :metadataFields 
    attr_reader :dbFields
    attr_reader :mappedFields
    attr_reader :tableFieldsMap
    attr_reader :fieldUINames
    attr_reader :aliases
    attr_reader :calculatedFields, :calculatedFieldsMap, :calculatedFieldNames, :calculatedField 
    attr_reader :allFields 
    attr_reader :groups 
    attr_reader :filters
    attr_reader :node

    def initialize dataSourceNode, workbook
      @node     = dataSourceNode
      @workbook = workbook 
      @name     = @node.attr('name')
      @caption  = @node.attr('caption')
      @uiname   = @caption.nil? ? @name : @caption
      # puts "DATASOURCE: #{@uiname}" 
      # processConnection
      # processFilters
      loadTableFields
      loadFieldUINames
      return self
    end

    def attributes
      @attributes ||= @attributes = loadAttributes(@node) 
    end

    def connAttributes
      @connAttributes ||= @connAttributes = loadAttributes(@connection)
    end

    def loadAttributes node
      attributes = {}
      unless node.nil?
        node.attributes.each do |k,v|
          attributes[k] = v.text
        end
      end
      return attributes
    end

    def id
        @id ||= @id = @name
    end

    def tableauVersion tabVersion
      @tableauVersion = tabVersion
    end

    def tableauVersion
      @tableauVersion
    end

    def connections
      @connections ||= @connections = processConnections
    end

    def processConnections
      @connections = Array.new
      processConnection './/connection'
      processConnection './/named-connection'
      return @connections
    end

    def processConnection path
        conns = @node.xpath(path)
        conns.each do |connNode|
          @connections << connNode
            # connClass = @dsclass
            # cpath     = connNode.path
            # connPath  = cpath.gsub(/\[[0-9]+\]/,'')
            # connPNum  = /(\d+)/.match(cpath)
            # # puts cpath, connPath
            # # puts "CPATH: #{cpath}"
            # conn.attributes.each do |name,value|
            #     # puts  "\n\t\t - %-15s -> %-s" % [name, value]
            #     $csvFile << [ $recCount += 1,
            #                   $twbName,  $dir,     $build,    $version,
            #                   $dsName,   $dstype,  $dsuiname, connClass, 
            #                   connPath,  connPNum,
            #                   name,      value.value
            #                 ]
            # end
        end
    end

    # def processConnection
    #   @connection  = @node.at_xpath('./connection')
    #   @dsclass     = @name  # handles 'Parameters' data source, which has no connection element (& others if so)  
    #   unless @connection.nil?
    #     @dsclass = @connection.attribute('class').text
    #     # note: must use "dsclass" as "class" would override Rubys ".class" Kernel method
    #     setConnectionHash
    #     loadTables @connection
    #   end
    # end

    # Notes:
    #      - TODO: need to determine which, if any, of the connection attributes should be
    #              included in the hash in order to identify it unambiguously - without
    #              local values that obscure the data source's 'real' identity
    #      - attributes with value '' don't contribute to the hash
    def setConnectionHash
      dsAttributes = @node.xpath('./connection/@*')
      dsConnStr    = ''
      dsAttributes.each do |attr|
        dsConnStr += attr.text
      end
      @connHash = Digest::MD5.hexdigest(dsConnStr)
      @uuid = @connHash
    end

    def isExtract
      @isExtract ||= @isExtract = !@node.at_xpath('./extract').nil?
    end

    def isPublished
      @isPublished ||= loadIsPublished
    end

    def loadIsPublished
      @isPublished = !node.at_xpath('./repository-location').nil?
    end

    def tables
      @tables ||= loadTables
    end

    def loadTables
      @tables = {}
      connections.each do |conn|
        nodes = conn.xpath(".//relation[@type='table']")
        nodes.each do |node|
          @tables[node.attr('name')] = node.attr('table')
        end
      end
      return @tables
    end

    def joinPairs
      @joinPairs ||= loadJoinPairs
    end

    def loadJoinPairs
      @joinPairs = Set.new
      mainJoin   = @node.xpath("./connection/relation[@type='join']")
      clauses    = @node.xpath(".//relation[@type='join']/clause") 
      clauses.each do |clause|
        leafs = clause.xpath('.//expression[not(node())]')
        @joinPairs << [ pullTable(leafs[0]) , pullTable(leafs[1]) ]
      end
      return @joinPairs
    end

    def joinTree
      @joinTree ||= loadJoinTree
    end

    def loadJoinTree
      loadJoinPairs if @joinPairs.nil?
      # puts  "LJT::#{@uiname}::joinPairs:: #{@joinPairs.inspect}"
      # @joinPairs.each { |jp| # puts "JP::#{jp}" }
      @joinTree = JoinTree.new(@name)
      @joinPairs.each do |from,to|
        # puts  "from:#{from} -> to:#{to}"
        tableFrom = JoinTable.new(from)
        tableTo   = JoinTable.new(to)
        @joinTree.add(tableFrom, tableTo)
      end
      # puts  '---'
      return @joinTree
    end

    def pullTable xml
        code =xml.attribute('op').text
        table = code.split('].[')[0][1..-1]
    end

    def Parameters?
      'Parameters'.eql? @name 
    end

    def columnField fieldName
      loadColumnFields if @columnFieldsMap.nil?
      @columnFieldsMap[ fieldName ]
    end


    def columnFields
      @columnFields ||= loadColumnFields
    end

    def columnFieldsMap
      loadColumnFields if @columnFieldsMap.nil?
      return @columnFieldsMap
    end

    def loadColumnFields
      @columnFields    = Set.new
      @columnFieldsMap = {}
      nodes = @node.xpath('.//column')
      nodes.each do |n|
        # puts " "
        # puts @columnFieldsMap.keys.inspect
        name = n.attribute('name').text.sub(/^\[/,'').sub(/\]$/,'')
        loaded = @columnFieldsMap.has_key? name
        if loaded
           # puts "LOADED" 
           field = @columnFieldsMap[name]
           # puts "#{field.class}"
           field.loadAttributes(n)
        else 
           field = Twb::ColumnField.new n, self
           @columnFieldsMap[field.uiname] = field
           @columnFieldsMap[field.name]   = field
        end
        # puts "name:#{name}: \t nn:#{field.name}:\t loaded?:#{loaded}  node:#{n}:" 
      end
      @columnFields = @columnFieldsMap.values
      return @columnFields
    end 

    def aliases
      @aliases ||= loadAliases
    end

    def loadAliases
      @aliases   = {}
      # puts $node.xpath('.//column/aliases/..').length
      cnt = 0
      @node.xpath('./column//aliases/..').each do |anode|
        # puts "anode:: #{anode}"
        # puts " path:: #{anode.path}"
        aliasMap = {}
        nameTech = anode.attribute('name').text.gsub(/^\[|\]$/,'')
        name     = fieldUIName nameTech
        if ':Measure Names'.eql? name
          # puts "processing Measure Names"
          anode.xpath('.//alias').each do |vnode|
            keyCode  = vnode.attribute('key').text.gsub(/^[#"%]|[#"%]$/,'')
            key      = Twb::CodedField.new(keyCode).name
            field    = fieldUIName key
            alia     = vnode.attribute('value').text
            # puts "  keyCode:: #{key}"
            # puts "  key    :: #{key}"
            # puts "  field  :: #{field}"
            # puts "  alias  :: #{alia}"
            aliasMap[field] = alia
          end
        else
          anode.xpath('.//alias').each do |vnode|
            key  = vnode.attribute('key').text.gsub(/^[#"]|[#"]$/,'')
            alia = vnode.attribute('value').text
            # puts " key :: #{key}"
            # puts "alias:: #{alia}"
            aliasMap[key] = alia
          end
        end
        @aliases[name] = aliasMap
      end
      emit "FIELD ALIASES: @aliases"
      return @aliases
    end

    def fieldHasAliases fieldName
      !aliases[fieldName].nil?
    end

    def fieldAliases fieldName
      aliases[fieldName]
    end

    def fieldAlias fieldName, value
      emit "fieldAlias:  #{fieldName.class}  -> #{value.class}" 
      loadAliases  if @aliases.nil?
      return value if @aliases.nil? || @aliases[fieldName].nil? # unless fieldHasAliases(fieldName)
      fldAlias = @aliases[fieldName][value]
      fldAlias.nil? ? value : fldAlias
    end

    def deAlias fieldName, fAlias
      fldAliases = aliases[fieldName]
      return fAlias if fldAliases.nil?
      dbFieldValue = fldAliases.key(fAlias)
      dbFieldValue.nil? ? fAlias : dbFieldValue
    end

    def localFields
      @localFields ||= loadLocalFields
    end

    def loadLocalFields
      @localFields = Set.new
      unless @connection.nil?  # Parameters has no connection node, & no local fields
        connClass    = @node.at_xpath('./connection').attribute('class').text
        fxpath       = case connClass
                       when 'dataengine' then './column'
                       when 'sqlserver'  then './column'
                       else                   './connection/relation/columns/column'
                       end
        nodes = @node.xpath(fxpath)
        nodes.each do |node|
          field = Twb::LocalField.new(node)
          @localFields << field
        end
      end
      return @localFields
    end

    def metadataFields
      @metadataFields ||= loadMetadataFields
    end

    def loadMetadataFields
      @metadataFields = Set.new
      unless @connection.nil?  # Parameters has no connection node, & no metadata fields
        # nodes = @node.xpath(".//metadata-record[@class='column']")
        # # note: there are other nodes "<metadata-record class='capability'>" whose nature is unclear
        # #       these nodes have no value for their <name node, so are not loaded
        nodes = @node.xpath("./connection//metadata-record[@class='column']")
        nodes.each do |node|
          field = Twb::MetadataField.new(node)
          field.source = :db
          @metadataFields << field
        end
        nodes = @node.xpath('./extract//metadata-record')
        nodes.each do |node|
          field = Twb::MetadataField.new(node)
          field.source = :extract
          @metadataFields << field
        end
      end
      return @metadataFields
    end

    def updateTime
      attr = @node.xpath('.//@update-time').first
      @updateTime = attr.nil? ? nil : attr.value
    end

    def fieldUIName fieldName
      loadFieldUINames if @fieldUINames.nil?
      @fieldUINames[fieldName].nil? ? fieldName : @fieldUINames[fieldName]
    end

    def fieldUINames
      loadFieldUINames if @fieldUINames.nil?
      return @fieldUINames
    end

    def tableFields
      @tableFieldsMap.values
    end

    def dbFieldsMap
      @tableFieldsMap
    end

    def dbFields
      @tableFieldsMap.values
    end

    def mappedFieldsMap
      loadTableFields if @tableFieldsMap.nil?
      return @tableFieldsMap
    end

    def mappedFields
      loadTableFields if @tableFieldsMap.nil?
      return @tableFieldsMap.values
    end

    def calculatedFieldsMap
      @calculatedFieldsMap ||= loadCalculatedFields
    end

    def calculatedFields
      loadCalculatedFields if @calculatedFieldsMap.nil?
      return @calculatedFieldsMap.values
    end

    def calculatedFieldNames
      loadCalculatedFields if @calculatedFieldsMap.nil?
      return @calculatedFieldsMap.keys
    end

    def calculatedField(name)
      loadCalculatedFields if @calculatedFieldsMap.nil?
      return @calculatedFieldsMap[name]
    end

    def loadCalculatedFields
      @calculatedFieldsMap = {}
      cfnodes = @node.xpath("./column[calculation]")
      cfnodes.each do |node|
        calcField = Twb::CalculatedField.new node, self
        @calculatedFieldsMap[calcField.uiname] = calcField
      end
      return @calculatedFieldsMap
    end

    def allFields
      return @allFields ||= loadAllFields
    end

    def loadAllFields
      @allFields = SortedSet.new
      # dbFields.each { |f| @allFields << f }
      # calculatedFields.each { |f| @allFields << f }
      @allFields.merge dbFields
      @allFields.merge calculatedFields
    end

    def has_field? fieldName
      dbFieldsMap.has_key? fieldName
    end

    def field fieldName
      dbFieldsMap[fieldName]
    end

    def fieldTable fieldName
      loadTableFields if @tableFieldsMap.nil?
      dbField = @tableFieldsMap[fieldName]
      return dbField.nil? ? nil : dbField.dbtable
    end

    # fields are unique in the data source by UI name
    def loadTableFields
      # puts  "DATA SOURCE FIELD TABLE LOAD"
      @tableFieldsMap = {}
      fieldNodes = @node.xpath('./connection/cols/map')
      fieldNodes.each do |fn|
        dbField = Twb::DbField.new(@uiname, fn, :map)
        @tableFieldsMap[dbField.uiname] = dbField
      end
      relTableNodes = @node.xpath('.//relation[@table]')
      relTableNodes.each do |relNode|
        table = relNode.attribute('name').text
        cols  = relNode.xpath('./columns/column')
        cols.each do |col|
          dbField = Twb::DbField.new(@uiname, col, :tableColumn, table)
          fldName = col.attribute('name')
          @tableFieldsMap[dbField.uiname] = dbField
        end
      end
    end

    def filters
      @filters ||= loadFilters
    end

    def groups
      @groups ||= loadGroups
    end

    private

    def loadFieldUINames
      @fieldUINames = {}
       # puts"metadataFields: #{metadataFields.length}"
      metadataFields.each do |fld|
         # puts" - name:%-45s   |  caption:%-45s   |  uiname:%-45s " % [ "'#{fld.name}'",  "'#{fld.caption}'", "'#{fld.uiname}'"]
        unless fld.name.nil?
          @fieldUINames[fld.uiname]    = fld.uiname
          @fieldUINames[fld.localName] = fld.uiname unless  fld.localName.nil?
          @fieldUINames[fld.name]      = fld.uiname
        end
      end
       # puts"localFields: #{localFields.length}"
      localFields.each do |fld|
         # puts" - name:%-45s   |  caption:%-45s   |  uiname:%-45s " % [ "'#{fld.name}'",  "'#{fld.caption}'", "'#{fld.uiname}'"]
        unless fld.caption.nil?
          @fieldUINames[fld.name]   = fld.caption
          @fieldUINames[fld.uiname] = fld.caption
        end
      end
       # puts"groups: #{groups.length}"
      groups.each do |fld|
         # puts" - name:%-45s   |  caption:%-45s   |  uiname:%-45s " % [ "'#{fld.name}'",  "'#{fld.caption}'", "'#{fld.uiname}'"]
        unless fld.caption.nil?
          @fieldUINames[fld.name]   = fld.caption
          @fieldUINames[fld.uiname] = fld.caption
        end
      end
       # puts"calculatedFields: #{calculatedFields.length}"
      calculatedFields.each do |fld| 
         # puts" - name:%-45s   |  caption:%-45s   |  uiname:%-45s " % [ "'#{fld.name}'",  "'#{fld.caption}'", "'#{fld.uiname}'"]
        unless fld.caption.nil?
          @fieldUINames[fld.name]   = fld.caption
          @fieldUINames[fld.uiname] = fld.caption
        end
      end
       # puts"columnFields: #{columnFields.length}"
      columnFields.each do |fld|
         # puts" - name:%-45s   |  caption:%-45s   |  uiname:%-45s " % [ "'#{fld.name}'",  "'#{fld.caption}'", "'#{fld.uiname}'"]
        unless fld.caption.nil?
          # @fieldUINames[fld.name]                          = fld.caption
          @fieldUINames[fld.name.gsub(/^[\[]|[\]]$/,'')]   = fld.caption
          @fieldUINames[fld.uiname]                        = fld.caption
        end
      end
      return @fieldUINames
    end

    def loadGroups
      @groups = []
      groupNodes = @node.xpath('.//group')
      groupNodes.each do |groupNode|
        groupField = Twb::GroupField.new(groupNode)
        @groups << groupField
      end
      return @groups
    end

    # def processFilters
    def loadFilters
      @filters = []
      sharedFilters = @workbook.node.xpath('./shared-views/shared-view[@name="excel.41269.481293159719"]//filter')
      sharedFilters.each do |sf|
        filter = QuickFilter.new(sf,@workbook)
        @filters << filter
      end
      return @filters
    end

    #     @filters << 
    #   if @filters.nil?
    #     @filters = {}
    #     fnodes = @node.xpath('./filter')
    #     fnodes.each do |fn|
    #       columnN = fn.attribute('column')
    #       unless columnN.nil?
    #         memberNodes = fn.xpath('.//groupfilter/@member')
    #         unless memberNodes.nil?
    #           members = []
    #           memberNodes.each do |m|
    #             members.push m.text
    #           end
    #           @filters[columnN.text] = members
    #         end
    #       end 
    #     end
    #   end
    # end


  end # class DataSource


  class JoinTablePair
    attr_reader :from, :to
    def initialize(from, to)
      raise ParamaterException.new("'From' cannot be nil.") if from.nil?
      raise ParamaterException.new("'To' cannot be nil.")   if to.nil?
      @from = from
      @to   = to
    end
    def to_s
      "#{@from} -> #{@to}"
    end
  end # class JoinTablePair


  class JoinTable
    include Comparable

    attr_reader   :name,     :datasource #, :str
    attr_accessor :children, :depth

    def initialize(name, datasource=nil)
      @name       = name
      @datasource = datasource
      # @str        = nil
      @children   = {}
      @depth      = 0
    end

    def to_s
      str = "[#{@datasource}].[#{@name}] :: (#{@depth}) :: #{@children.length} :: "
      @children.each { |n,c| str << "#{n}, " }
      return str
    end

    def <=>(anOther)
      @str <=> anOther.str
    end

    def addChild child
      # puts  "#{@name}.addChild(#{child.name})"
      # puts  "children: #{@children}"
      @children[child.name] = child if @children[child.name].nil?
      # puts  "children: #{@children}"
    end

    def child name
      @children[name]
    end

  end # class JoinTable


  class JoinTree

    attr_reader  :datasource, :root, :maxdepth, :tables

    def initialize  datasource
      @datasource = datasource
      @root       = nil
      @maxdepth   = 0
      @tables     = {}
    end

    def add host, dest
      # puts  "\nJT add() host: #{host}"
      # puts  "           dest: #{dest}"
      from = @tables[host.name].nil? ? host : @tables[host.name]
      to   = @tables[dest.name].nil? ? dest : @tables[dest.name]
      from.addChild(to)
      @tables[from.name] = from
      @tables[to.name]   = to
      if @root.nil?  || @root.name.eql?(to.name)
        @root = from
      end
      setDepth(@root,1)
    end

    def table tableName
      @tables[tableName]
    end

    def setDepth table, depth
      # puts  "-- setDepth(#{table.class}[#{table.name}] \t, #{depth})"
      @tables[table.name].depth = depth
      childrenDepth = depth+1
      table.children.each { |n,c| setDepth(c,childrenDepth)}
    end

    def tableDepth tableName
      @tables[tableName].nil? ? 0 : @tables[tableName].depth
    end

    def disp table
      # puts  "%s %s %s  (%d)" % [' ' * table.depth, '-' * table.depth, table.name, table.depth]
      table.children.each { |n,c| disp c}
    end

    def iterate
      disp @root
    end

    def to_s
      str = "root: #{@root}\ntables"
      @tables.each do |t| 
        str << " tbl:: #{t}"
      end
      return str
    end

  end # class JoinTree  

end # module Twb
