require 'nokogiri'
require 'csv'
require 'twb'

def init
  system 'cls'
  $pFile = File.open('identifyFields.txt','w')

  sqiCSV = 'TWBFieldsByType.csv'
  $fieldsCSV = CSV.open(sqiCSV, 'w')
  $fieldsCSV    << [
                    'TWB', 
                    'Data Connection - UI', 
                    'Field Name', 
                    'TWB Field Type', 
                    'Data Type', 
                    'Calc Field?', 
                   ]
  $path    = if ARGV.empty? then '**/*.twb' else ARGV[0] end
  emit " "
  emit " Identifying fields in Tableau Workbook Data Connections (TWDCs).\n "
  emit " Looking for Workbooks matching: '#{$path}'  - from: #{ARGV[0]}\n\n "
end

$localEmit = true
def emit stuff
    $pFile.puts stuff
           puts stuff if $localEmit
end

$paths = { 
          'Relation Column' => { 'fieldXPath' => './connection/relation/columns/column',
                                 'nameXPath'  => './@name' 
                               },
          'Metadata Record' => { 'fieldXPath' => './connection/metadata-records/metadata-record[@class="column"]',
                                 'nameXPath'  => './remote-name[text()]'
                               },
          'Columns'         => { 'fieldXPath' => './column',
                                 'nameXPath'  => './@name' 
                               },
         }

# different connection classes have 
# different paths & attributes to the Relation fields
$relClassPaths = { 
                   'dataengine'   => { 'fieldXPath' => './connection/relation/columns/map',
                                       'nameXPath'  => './@name'
                                     },
                   'excel-direct' => { 'fieldXPath' => './connection/relation/columns/map',
                                       'nameXPath'  => './@name'
                                     },
                   'textscan'     => { 'fieldXPath' => './connection/relation/columns/map',
                                       'nameXPath'  => './@name'
                                     },
                  }

def relationFields dataSource, connClass
  emit "\t relationFields  connClass: '%s'" % [connClass]
  xPaths     = $relClassPaths[connClass]
  emit "\t xPaths: #{xPaths}"
  fnodesPath = xPaths['fieldXPath']
  fnamePath  = xPaths['nameXPath']
  fieldNodes = dataSource.xpath(fnodesPath)
    
end

def metaDataRecords dataSource
  emit "\t metaDataRecords  " #connClass: '%s'" % [connClass]
  mdNodes = dataSource.xpath('./connection/metadata-records/metadata-record[@class="column"]')
  emit "  mdNodes.size: #{mdNodes.size}"
  fields = {}
  mdNodes.each do |node|
    name = node.xpath('./local-name').text.gsub(/^\[/,'').gsub(/\]$/,'')
    type = node.xpath('./local-type').text
    fields[name] = {'type' => type}
  end
  return fields
end

def localColumns dataSource
  emit "\t localColumns  " #connClass: '%s'" % [connClass]
  colNodes = dataSource.xpath('./column')
  emit "  colNodes.size: #{colNodes.size}"
  fields = {}
  colNodes.each do |node|
    calcField = !node.at_xpath('./calculation').nil?
    caption   = node.attribute('caption')
    name      = node.attribute('name').text.gsub(/^\[/,'').gsub(/\]$/,'')
    type      = node.attribute('datatype')
    emit "  \t    name: %s\n \t    type: %s\n \t caption: %s" % [name, type, caption]
    caption   = name if caption.nil?
    emit "  \t caption: %s  \n " % [caption]
    fields[caption] = {'type' => type, 'calcField' => calcField}
  end
  return fields
end

def process file
  emit "\n == #{file}"
  doc = Nokogiri::XML(open(file))
  dataSourcesNodes = doc.xpath('//workbook/datasources/datasource')
  dataSourcesNodes.each do |ds|
    dsCaption = ds.attribute('caption')
    dsName    = ds.attribute('name')
    emit "\n  dsCapt: #{dsCaption}"
    emit   "  dsName: '#{dsName}'\n   ---"
    connClass = ds.xpath('./connection/@class')
    emit   "dsCClass: '#{connClass}'\n   ---"
    if dsName != 'Parameters' then
      # relationFields  ds, connClass
      mdFields = metaDataRecords ds
      mdFields.each do |fname, payload|
        emit "\t "
        $fieldsCSV << [file, dsCaption, fname, 'metadata', payload['type'] ]
        emit  [file, dsCaption, fname, 'metadata', payload['type'] ]
      end
      colFields = localColumns  ds
      colFields.each do |fname, payload|
        emit "\t "
        $fieldsCSV << [file, dsCaption, fname, 'column', payload['type'], payload['calcField'] ]
        emit          [file, dsCaption, fname, 'column', payload['type'], payload['calcField'] ]
      end
    end

#    $paths.each do |type, paths|
#      emit "    type: #{type}"
#      emit paths['fieldXPath']
#      nodes = ds.xpath(paths['fieldXPath']).to_a
#      nodes.each do |node|
#          name = node.xpath(paths['nameXPath']).text
#          emit "   fname: #{name}"
#      end
#    end
  end
end

init

def showPaths
  $paths.each do |type, paths|
  	emit "  #{type}"
    paths.each do |key, value|
      emit "      : %10s  =>  %-s" % [key, value]    
    end
    puts " "
  end
end

emit "XXX \n#{$relClassPaths}"

Dir.glob($path) {|twb| process twb }

emit "\n\n== Done ==\n "
