#  Copyright (C) 2014, 2018  Chris Gerrard
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

require 'nokogiri'

module Twb

  class DbField < TabClass

    include Comparable

    attr_reader :node,       
    attr_reader :properties
    attr_reader :name,       :uiname,        :dbname
    attr_reader :dbtable

    def initialize dataSource, fieldNode
      @node    = fieldNode
      @uiname  = @node.attribute('key').text.gsub(/^\[|\]$/,'')
      @name    = @uiname
      fldRef   = @node.attribute('value').text.gsub(/^\[|\]$/,'')
      parts    = fldRef.split('].[')
      @dbtable = parts[0]
      @dbname  = parts[1]
      @id      = "#{dataSource}::#{dbname}::#{dbtable}::#{uiname}"
    end

    def id
        @id ||= @id = @name.hash
    end

    def properties
      @properties ||= loadProperties
    end

    def loadProperties
      @properties= {}
      @node.attributes.each do |attr|
        @properties[attr.name] = attr.text
      end
      @properties[:uiname] = @name
    end

    def to_s
      @id
    end

    def <=>(other)
      @id <=> other.id
    end

  end  # class ColumnField

end
