#  Copyright (C) 2014, 2015  Chris Gerrard
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

require 'nokogiri'

module Twb

  class MetadataField < TabClass

    include Comparable

    attr_reader     :node,         :properties
    attr_reader     :twbClass
    attr_accessor   :source
    attr_reader     :parentName,   :table        # parentName is the Db table
    attr_reader     :localName,    :name        
    attr_reader     :caption,      :uiname
    attr_reader     :localType,    :family
    attr_reader     :remoteName,   :remoteAlias  
    attr_reader     :remoteType,   :aggregation 
    attr_reader     :containsNull, :ordinal
    attr_reader     :precision,    :width,       :scale

    # Child nodes of <metadata-record nodes 
    # <aggregation
    # <approx-count
    # <attributes
    # <caption
    # <collation
    # <contains-null
    # <family
    # <layered
    # <local-name
    # <local-type
    # <ordinal
    # <parent-name
    # <precision
    # <remote-alias
    # <remote-name
    # <remote-type
    # <scale
    # <statistics
    # <width


    def initialize fieldNode
      @node              = fieldNode
      @twbClass          = @node.attribute('class').text
      @parentName        = load 'parent-name'
      @family            = load 'family'
      @table             = @family.nil? ? @parentName : @family
      @remoteName        = load 'remote-name' 
      @remoteAlias       = load 'remote-alias'
      @remoteType        = load 'remote-type'
      @caption           = load 'caption'   
      @localName         = load 'local-name'
      @name              = @localName
      @uiname            = @caption.nil? ? @localName : @caption
      @aggregation       = load 'aggregation'
      @containsNull      = load 'contains-null'
      @localType         = load 'local-type'
      @ordinal           = load 'ordinal'
      @precision         = load 'precision'
      @scale             = load 'scale'
      @width             = load 'width'
      @id                = "'%s::%s' " % [@table,@remoteName]   
    end

    def id
        @id ||= @id = @name.hash
    end

    def properties
      @properties ||= loadProperties
    end

    def loadProperties
      @properties = {}
      children = @node.xpath('child::node()')
      children.each do |child|
        @properties[child.name] = child.text unless 'attributes'.eql? child.name
      end
      @properties[:uiname] = @uiname
      @node.attributes.each do |name,attr|
        @properties[name] = attr.value
      end
      return @properties
    end

    def load nodeName
      node = @node.at_xpath(nodeName)
      val = node.nil? ? nil : node.text.strip.gsub(/^\[|\]$/,'')
      return val
    end

    def to_s
      @id
    end

    def <=>(other)
      @id <=> other.id
    end

  end

end



