#  calculatedfieldsanalyzer.rb - this Ruby script Copyright 2017, 2018 Christopher Gerrard
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

# require 'nokogiri'
# require 'twb'
# require 'set'
require 'csv'

module Twb
module Util

  class AtomizeCSVRecords

      include TabTool
      # include Graph

      attr_reader   :metrics
      attr_accessor :ttdocdir

      @@ttlogfile   = 'AtomizeCSVRecords.ttlog'
      @@csvFileName = 'AtomizedCSVRecords.csv'  

      def initialize(**args)
          @ttdocdir  = @args[:ttdocdir]
          @csvAdd    = @args[:csvMode] == :add
          @csvMode   = @csvAdd ? 'a' : 'w'
          init
          @funcdoc   = {:class=>self.class, :blurb=>'Analyze Calculated Fields', :description=>'Calculated fields can be complex, this tool provides robust coverage.',}
          # --
          @executed  = false
      end

      def loadFields fileName
        fields = Set.new
        file = File.open(fileName)
        unless file.nil?
          file.each_line { |line|
            fields.add line.chomp
          }
        end
        return fields
      end

      def processTWB workbook
        puts "\n\t ALERT: Utility tool: #{self.class} - does not implement Workbook processing"
        puts   "\t                                    - deferring to single execution"
        processData fileName
      end

      def processData fileName
        unless @executed
          emit true, "initialize AtomizeCSVRecords args #{args}"
          @atomicCSV = CSV.open(@@csvFileName, 'w')
          # --
          inputFileName = fileName.nil? ? 'csvData.csv' : fileName
          keyFields     = loadFields 'key.fields'
          excludeFields = loadFields 'exclude.fields'
          puts "\t Using these key fields: #{$keyFields.inspect}"
          puts "\t Excluding these fields: #{$excludeFields.inspect}"

            $fieldValsCSV = '.csv'
            $atomicCSV    = CSV.open($fieldValsCSV, 'w')
            $atomicFields = $keyFields.to_a + [ 'Rec #', 'Field', 'Value' ]
            puts "\n\t CSV output fields: #{$atomicFields.inspect}"
            $atomicCSV << $atomicFields


            $recCnt, $fldsCnt = 0, 0

            $recNum = 0

          @executed = true
        end
      end

      def metrics
        @metrics ||= loadMetrics
      end

      def loadMetrics
        @metrics = {
                     '# of Data Sources'      => @dataSourcesCount,
                     '# of Calculated Fields' => @calculatedFieldsCount,
                     '# of Referenced Fields' => @referencedFieldsCount,
                   }
      end

      #-- private methods begin here, to end of class

  end #  class

end   # module Analysis
end   # module Twb