#  CSVRecordsAtomizer.rb - this Ruby script Copyright 2019 Christopher Gerrard
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

require 'csv'

module Twb
module Util

  class CSVRecordsAtomizer

      include TabTool

      attr_reader   :metrics
      attr_accessor :ttdocdir

      @@ttlogfile    = 'AtomizeCSVRecords.ttlog'
      @@csvFileName  = 'AtomizedCSVRecords.csv'
      @@atomFields   = ['Record #', 'Field Name', 'Field Value'] 

      def initialize(**args)
          @args      = args
          @ttdocdir  = @args[:ttdocdir]
          @csvAdd    = @args[:csvMode] == :add
          @csvMode   = @csvAdd ? 'a' : 'w'
          init
          @funcdoc   = {:class=>self.class, :blurb=>'Analyze Calculated Fields', :description=>'Calculated fields can be complex, this tool provides robust coverage.'}
          # --
          @executed  = false
      end

      def processFile fileName
          emit true, "#{self.class} - processFile: #{fileName} :: #{fileName.class}"
            # --
          srcFields = Set.new CSV.open(fileName, 'r:bom|utf-8', &:readline)
          emit true, "\t Inspecting these #{srcFields.length} #{srcFields.class} fields: "
          keyFields     = loadFields(fileName, :keyfields)
          excludeFields = loadFields(fileName, :excludefields)
          recordFields  = (srcFields - keyFields - excludeFields).to_a
          emit true, "\t       Using these key fields: #{keyFields.inspect}"
          emit true, "\t       Excluding these fields: #{excludeFields.inspect}"
          emit true, "\t       Recording these fields: #{recordFields.inspect}"
          # --
          atomizedFields  = keyFields.to_a + @@atomFields
          dataBaseName    = File.basename(fileName, ".*")
          atomizedCSVFile = "#{dataBaseName}.Atomized.csv" 
          atomizedCSVFile = CSV.open(atomizedCSVFile,'w') 
          atomizedCSVFile << keyFields.to_a + @@atomFields
          # --
          @recNum = 0
          CSV.open(fileName, 'r:bom|utf-8', headers: true) do |csv|
            csv.each do |row|
              @recNum += 1
              atomRec = []
              keyFields.each do |kf|
                atomRec << row.fetch(kf)
              end
              atomRec << @recNum
              recordFields.each do |rf|
                atomizedCSVFile << atomRec + [rf, row.fetch(rf)] 
              end
            end
          end
      end

      private

      def loadFields fileName, type
        puts "\n\n\t loadFields -#{fileName}-  -#{type}-"
        fields    = Set.new
        auxFile   = "#{fileName}.#{type}"
        isAuxFile = File.exist?(auxFile)
        emit true, "\t\t Loading auxilary data: %-12s %7s %-s" % [type, isAuxFile, auxFile]
        if isAuxFile
          # file = File.open(fileName)
          File.open(auxFile).each_line { |line|
            fields.add line.chomp
          }
        end
        return fields.to_a
      end

      def processData fileName
      end

      def metrics
        @metrics ||= loadMetrics
      end

      def loadMetrics
        @metrics = {
                     '# of Data Sources'      => @dataSourcesCount,
                     '# of Calculated Fields' => @calculatedFieldsCount,
                     '# of Referenced Fields' => @referencedFieldsCount,
                   }
      end

      #-- private methods begin here, to end of class

  end #  class

end   # module Analysis
end   # module Twb