#  Copyright (C) 2012, 2015  Chris Gerrard
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

require 'digest/md5'

module Twb
module Util

  class GML

    include TabTool

    @@hasher = Digest::SHA256.new

    @@docfileName = './ttdoc/gml.log'

#====================================================================
@@gmlHeader = <<GMLHEADER
graph [
  directed 1
  rankdir "LR"
  splines "line"

GMLHEADER

@@gmlFooter = "\n]"

    attr_accessor :nodes, :edges, :fileName, :gmlHeader

    def initialize
      # @docFile = File.open(@@docfileName,'w')
      # @docFile.puts "Starting up the GML process"
      @gmlHeader = @@gmlHeader
      @fileName  = 'gmlFile'      # will be reset in def render when file is generated
      @nodes     = SortedSet.new
      @edges     = SortedSet.new
    end

    # def gmlHeader=(text)
    #   @gmlHeader = text
    # end

    # def init fileName 
    #   # do stuff for new GML file here
    #   @fileName = fileName
    #   prepare
    # end

    def prepare
      @nodes = SortedSet.new
      @edges = SortedSet.new
    end

    def render
      # puts "GML.render - rendering into #{@fileName}"
      @fileName = "#{@fileName}.gml"
      file = File.open(docFile(@fileName),'w')
      file.puts @gmlHeader
      renderNodes file
      renderEdges file
      file.puts @@gmlFooter
      file.close
      return file
    end

    def renderNodes file
      nodes = Set.new
      # puts 'def renderNodes'
      @nodes.each do |node|
        gmlID   = Digest::MD5.hexdigest(node.id)
        gmlName = node.name.gsub('&','&amp;').gsub('"','&quot;')
        nodestr = "node [\n        id    \"#{gmlID}\" \n        label \"#{gmlName}\" "
        unless node.colour.nil?
          nodestr += "\n    graphics \n   [ \n      fill  \"#{node.colour}\" \n   ] "
        end
        nodestr += "\n    ]"
        nodes << nodestr
      end
      nodes.each do |node|
        file.puts node
      end
    end

    def renderEdges file
      edges = Set.new
      @edges.each do |edge|
        gmlSourceID = Digest::MD5.hexdigest(edge.from.id)
        gmlTargetID = Digest::MD5.hexdigest(edge.to.id)
        edges << "edge [\n        source \"#{gmlSourceID}\" \n        target \"#{gmlTargetID}\" \n    ]"
      end
      edges.each do |edge|
        file.puts edge
      end
    end

    def to_s
      "file:#{@fileName}; #nodes:#{@nodes.length}; #edges:#{@edges.length}"
    end


  end

end  # module Util
end  # module Twb


    # node [
    #   id 7
    #   label "5"
    #   edgeAnchor "corners"
    #   labelAnchor "n"
    #   graphics [
    #     center [ x 82.0000 y 42.0000 ]
    #     w 16.0000
    #     h 16.0000
    #     type "rectangle"
    #     fill "#000000"
    #   ]
    # ]
    # edge [
    #   label "24"
    #   labelAnchor "first"
    #   source 7
    #   target 15
    #   graphics [
    #     type "line"
    #     arrow "last"
    #     Line [
    #       point [ x 82.0000 y 42.0000 ]
    #       point [ x 10.0000 y 10.0000 ]
    #       point [ x 100.000 y 100.000 ]
    #       point [ x 80.0000 y 30.0000 ]
    #       point [ x 120.000 y 230.000 ]
    #       point [ x 73.0000 y 160.000 ]
    #     ]
    #   ]
    # ]


