#  Copyright (C) 2018  Chris Gerrard
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

module Twb
module Util

  class GMLedge

    # @from - the origin node
    # @to   - the destination node
    # @relationship - useful for categorizing the edge
    # @properties   - useful for categorizing the edge
    attr_reader   :from, :to, :relationship
    attr_accessor :properties
    attr_reader   :cypherCreate

    # Neo4J cypher variable quote character: `  

    def initialize (from:, to:, relationship:, properties: {})
      raise ArgumentError.new("from: parameter must be a Graphnode, is a '#{from.class}'") unless from.is_a? Twb::Util::Graphnode
      raise ArgumentError.new(  "to: parameter must be a Graphnode, is a '#{to.class}'"  ) unless   to.is_a? Twb::Util::Graphnode
      @from         = from
      @to           = to
      @relationship = relationship
      @properties   = properties
      @cypherCreate = "CREATE  #{cypher_s}"
    end

    def eql? other
      @from == other.from && @to == other.to && @relationship == other.relationship && @properties == other.properties
    end

    def hash
      [@from.hash, @to.hash, @relationship, @properties].hash
    end

    def to_s
      "'#{@from.name}//{@from.id}' --#{@relationship}--> '#{@to.name}//#{@to.id}'"
    end

    def dot
      "%s  ->  %s" % [dotquote(from.id), dotquote(to.id)]
    end

    def dotquote str
      ns = str.gsub(/(["])/,'\\"')
      return "\"#{ns}\""
    end

    def cypher_s
      "(%s)-[:`%s`]->(%s)" % [@from.cypherID,@relationship,@to.cypherID]
    end

    def gml
      "%s  ->  %s" % [dotquote(from.id), dotquote(to.id)]
    end

  end

end  # module Util
end  # module Twb
