#  Copyright (C) 2014, 2017  Chris Gerrard
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

require 'securerandom'

module Twb
module Util

  class Graphnode #< TabClass

    include Comparable

    @@hasher = Digest::SHA256.new
 
    @@stripChars = /[.: %-\-\(\)=]/
    @@replChar   = '_'

    @@typeShapes = { :CalculatedField   => 'shape=note',
                     :DBTable           => 'shape=ellipse',
                     :TwbDataConnection => '',
                     :DatabaseField     => ''
                   }

    @@typeColors = { :CalculatedField   => 'fillcolor=lightskyblue3',
                     :DBTable           => 'fillcolor=steelblue',
                     :TwbDataConnection => 'fillcolor=cornflowerblue',
                     :DatabaseField     => 'fillcolor=grey',
                   }

    @@typeStyles = { :CalculatedField   => 'style=filled',
                     :DBTable           => 'style=filled',
                     :TwbDataConnection => 'style=filled',
                     :DatabaseField     => 'style=filled'
                   }

    attr_reader   :id, :uuid, :type, :name, :dotid
    attr_accessor :properties, :colour

    def initialize (name:, id:, type:, properties: {})
      # puts "graphNode : t: %-20s  tc: %s " % [type, type.class]
      @id           = id
      @uuid         = SecureRandom.uuid
    # @type         = type.instance_of?(Class) ? type      : type.class
      @type         = resolveType type 
      # puts "          : t: %-20s  tc: %s \n " % [@type, @type.class]
      @name         = name
      @colour       = nil
      @properties   = properties
      @properties['name'] = name unless @properties.key? 'name'
      if @name.nil? 
        raise ArgumentError, "Graphnode Initialize Error - name:'#{@name}' id:'#{@id}'  type:'#{type}' properties:'@properties.inspect'"
      end
    end

    def resolveType type
         if type.instance_of?(Class) 
              deModule(type)
      elsif type.instance_of?(Symbol) 
              type.to_s 
      elsif type.instance_of?(String)
              type
      else  deModule(type)
      end
    end

    def deModule fqClass
      fqClass.class.name.split('::').last
    end

    def enquote str
      str.gsub("'","\\\\'")
    end

    def dotLabel
      "%s [label=\"%s\"  %s %s ]" % [dotid, name.gsub('"','\"'), @@typeShapes[@type], @@typeColors[@type]] 
    end

    def dotid
      @dotid ||= @dotid = dotquote(@id)
    end

    def dotquote str
      ns = str.instance_of?(String) ? str.gsub(/(["])/,'\\"') : str.to_s
      return "\"#{ns}\""
    end

    def eql? other
      @name == other.name && @id == other.id && @type == other.type  && @properties == other.properties
    end

    def hash
      [@name, @id, @type, @properties].hash
    end

    # def uuid
    #   @uuid ||= @uuid = SecureRandom.uuid
    #   # @uuid ||= Digest::MD5.hexdigest(id)
    # end

    def to_s
      "name:'%s'  id:'%s'  t:'%s'  p:%s" % [@name, @id, @type, @properties.to_s]
    end

    def props_s
      @props_s ||= @properties.map{|k,v| "#{k}: #{v.inspect}"}.join(', ')
    end

    def <=>(other)
      hash <=> other.hash
    end

  end # class Graphnode

end  # module Util
end  # module Twb
