#  Copyright (C) 2012, 2015  Chris Gerrard
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

module Twb
module Util

  class TwbDashesSheetDataDotBuilder

    @@dotHeader = <<-DOTHEADER
    digraph g {
        graph [rankdir="LR" splines="line"];
        node  [shape="box"  width="2"];

        "Tableau Tools generated Workbook map" [color="white" border="0"];
    DOTHEADER

    @@dotFooter = <<-DOTFOOTER
      }

       subgraph cluster_0 {
        color=white;
        node [shape="box3d"  style="filled" ];
        "Workbook" -> "Dashboard" -> "Worksheet" -> "Data Source"
        "Workbook"    "Dashboard";
        "Worksheet";
        "Data Source";
       }

    }
    DOTFOOTER

    attr_reader :workbook, :dashesDotFiles, :dotFileName, :dashCount, :sheetCount, :dataSourceCount

    def initialize twb
      @workbook = if twb.instance_of?(Twb::Workbook)
                  then twb
                  else Twb::Workbook.new(twb)
                  end
      @dashboards =  @workbook.dashboards
      @dashboards.each do |dash|
        initDot
      end
      buildBody(pairs)
      sameRank([@workbook.name]           )
      sameRank( @workbook.dashboardNames  )
      sameRank( @workbook.worksheetNames  )
      sameRank( @workbook.datasourceNames )
      buildHeader
      labelDataSources
      closeDot
    end

    def initDot
      @dotFileName = @workbook.name + '.dot'
      @dotFile = File.open(@dotFileName,'w')
      @dotFile.puts @@dotHeader
    end

    def buildBody(pairs)
      @dotFile.puts "\n   subgraph cluster_1 {"
      @dotFile.puts "       color= grey;"
      @dotFile.puts ""
      pairs.each { |pair| @dotFile.puts "      \"#{pair[0]}\" -> \"#{pair[1]}\" " }
      @dotFile.puts ""
      @dotFile.puts "   }"
    end

    def sameRank(elements)
      @dotFile.puts "\n  {rank=same "
      elements.each do |e|
        @dotFile.puts "     \"#{e}\""
      end
      @dotFile.puts "  }"
    end

    def buildHeader
     @dotFile.puts ''
     @dotFile.puts '   subgraph cluster_0 {'
     @dotFile.puts '     color=white;'
     @dotFile.puts '     node [shape="box3d"  style="filled" ];'
     @dotFile.puts '     "Workbook" -> "Dashboard" -> "Worksheet" -> "Data Source"'
     @dotFile.puts '   }'
    end

    def labelDataSources
      @dotFile.puts " "
      @workbook.datasources.each do |ds|
        name = ds.name
        @dotFile.puts "  \"#{name}\" [label=\"#{ds.uiname}\"];"
      end
    end

    def closeDot
     @dotFile.puts ' '
     @dotFile.puts '}'
     @dotFile.close
    end

    def processDashboards
      pairs = []
      @workbook.dashboards.each do |dash|
        dashName = dash.name
        pairs          << [@workbook.name,dashName]
        dash.worksheets.each do |sheet|
          sheetName = sheet.name
          pairs << [dashName,sheetName]
          @worksheets.delete sheetName
        end
      end
      return pairs
    end

    def processWorksheets
      pairs = []
      @workbook.worksheets.each do |sheet|
        sheetName   = sheet.name
        sheet.datasources.each do |ds|
          dsName = ds.name
          pairs  << [sheetName,ds.name]
          @datasources.delete ds.uiname
        end
      end
      return pairs
    end

    def processOrphans
      pairs = []
      @datasources.each { |dsn| pairs << [@workbook.name,dsn] }
      @worksheets.each  { |wsn| pairs << [@workbook.name,wsn] }
      return pairs
    end

  end

end
end